"""Register CLI commands for user."""
import argparse
import os

from inductiva._cli import loader, utils
from inductiva import constants


def register(root_parser):
    parser = root_parser.add_parser(
        "user",
        help="User management utilities.",
        formatter_class=argparse.RawTextHelpFormatter)

    parser.description = ("User management utilities.\n\n"
                          "The `inductiva user` command allows you to "
                          "consult your user's internal information.\n")

    utils.show_help_msg(parser)

    subparsers = parser.add_subparsers(title="available subcomands")
    loader.load_commands(subparsers,
                         os.path.dirname(__file__),
                         package=__name__,
                         ignores_prefix=constants.LOADER_IGNORE_PREFIX)



"""Base class for machine groups."""
from collections import defaultdict, namedtuple
import time
import enum
import json
from typing import Optional, Union
from abc import abstractmethod
import datetime

import logging

import inductiva
import inductiva.client.models
from inductiva import api, users
from inductiva.utils import format_utils
from inductiva.client.apis.tags import compute_api

from inductiva.resources import machine_types

VCPUCount = namedtuple("VCPUCount", ["total", "per_machine"])


class ResourceType(enum.Enum):
    """Enum to represent the type of machine to be launched."""

    STANDARD = "standard"
    MPI = "mpi"


class BaseMachineGroup:
    """Base class to manage Google Cloud resources."""

    def __init__(
        self,
        machine_type: str,
        provider: Union[machine_types.ProviderType, str] = "GCP",
        threads_per_core: int = 2,
        data_disk_gb: int = 10,
        max_idle_time: Optional[datetime.timedelta] = None,
        auto_terminate_ts: Optional[datetime.datetime] = None,
        register: bool = True,
    ) -> None:
        """Create a BaseMachineGroup object.

        Args:
            machine_type: The type of GC machine to launch. Ex: "e2-standard-4".
              Check https://cloud.google.com/compute/docs/machine-resource for
              more information about machine types.
            provider: The cloud provider of the machine group.
            data_disk_gb: The size of the disk for user data (in GB).
            threads_per_core: The number of threads per core (1 or 2).
            max_idle_time: Time without executing any task, after which the
              resource will be terminated.
            auto_terminate_ts: Moment in which the resource will be
              automatically terminated.
            register: Bool that indicates if a machine group should be register
                or if it was already registered. If set to False by users on
                initialization, then, the machine group will not be able to be
                started. This serves has an helper argument for retrieving
                already registered machine groups that can be started, for
                example, when retrieving with the `machines_groups.get` method.
                Users should not set this argument in anyway.
        """

        provider = machine_types.ProviderType(provider)
        self.provider = provider.value

        if data_disk_gb <= 0:
            raise ValueError("`data_disk_gb` must be positive.")

        if threads_per_core not in [1, 2]:
            raise ValueError("`threads_per_core` must be either 1 or 2.")

        self.machine_type = machine_type
        self.provider = provider.value
        self.threads_per_core = threads_per_core
        self.data_disk_gb = data_disk_gb
        self._id = None
        self._name = None
        self.create_time = None
        self._started = False
        self.register = register
        #Number of active machines at the time of
        #the request machine_groups.get()
        self._active_machines = 0
        self.num_machines = 0
        self._max_idle_time = max_idle_time
        self._auto_terminate_ts = auto_terminate_ts

        # Set the API configuration that carries the information from the client
        # to the backend.
        self._api = compute_api.ComputeApi(api.get_client())
        self._estimated_cost = None

    @property
    def id(self):
        return self._id

    @property
    def n_vcpus(self):
        """Returns the number of vCPUs available in the resource.

        Returns a tuple with the total number of vCPUs and the number of vCPUs
        per machine. For a machine group with 2 machines, each with 4 vCPUs,
        this will return (8, 4).
        In a case of an Elastic machine group, the total number of vCPUs is
        the maximum number of vCPUs that can be used at the same time.
        """
        max_vcpus = int(self.quota_usage["max_vcpus"])
        max_instances = int(self.quota_usage["max_instances"])

        return VCPUCount(max_vcpus, max_vcpus // max_instances)

    @property
    def name(self):
        return self._name

    @property
    def max_idle_time(self) -> datetime.timedelta:
        return self._max_idle_time

    @property
    def auto_terminate_ts(self):
        return self._auto_terminate_ts

    @property
    def idle_time(self) -> datetime.timedelta:
        """
        Resource idle time in seconds.
        """
        return self._idle_seconds

    @staticmethod
    def _timedelta_to_seconds(
            value: Optional[datetime.timedelta] = None) -> Optional[float]:
        """Converts a timedelta object to seconds."""
        return value.total_seconds() if value is not None else None

    @staticmethod
    def _seconds_to_timedelta(
            value: Optional[float] = None) -> Optional[datetime.timedelta]:
        """Converts seconds to a timedelta object."""
        return datetime.timedelta(
            seconds=float(value)) if value is not None else None

    @staticmethod
    def _convert_auto_terminate_ts(
            timestamp: Optional[datetime.datetime]) -> Optional[str]:
        """Converts a datetime object to ISO format. Additionally, it checks
        if the datetime is in the future and if it is timezone aware."""
        if timestamp is not None:
            if (timestamp.tzinfo is None or
                    timestamp.tzinfo.utcoffset(timestamp) is None):
                raise ValueError("auto_terminate_ts must be timezone aware.")

            now_ts = datetime.datetime.now(timestamp.tzinfo)
            if timestamp < now_ts:
                raise ValueError("auto_terminate_ts must be in the future.")
            return timestamp.isoformat()

        return None

    @staticmethod
    def _iso_to_datetime(
            timestamp: Optional[str]) -> Optional[datetime.datetime]:
        """Converts an ISO format string back to a datetime object. It ensures
        the datetime is timezone aware."""
        if timestamp is not None:
            dt = datetime.datetime.fromisoformat(str(timestamp))
            if dt.tzinfo is None or dt.tzinfo.utcoffset(dt) is None:
                raise ValueError("The datetime string must be timezone aware.")
            return dt
        return None

    def _register_machine_group(self, **kwargs):
        """Register machine group configuration in API.

        Returns:
            The unique ID and name identifying the machine on the API."""

        instance_group_config = inductiva.client.models.VMGroupConfig(
            machine_type=self.machine_type,
            provider_id=self.provider,
            threads_per_core=self.threads_per_core,
            disk_size_gb=self.data_disk_gb,
            max_idle_time=self._timedelta_to_seconds(self.max_idle_time),
            auto_terminate_ts=self._convert_auto_terminate_ts(
                self.auto_terminate_ts),
            **kwargs,
        )

        resp = self._api.register_vm_group(
            body=instance_group_config,
            skip_deserialization=True,
        )
        body = json.loads(resp.response.data)

        self._id = body["id"]
        self._name = body["name"]
        self.quota_usage = body.get("quota_usage") or {}
        self.register = False
        # Lifecycle configuration parameters are updated with default values
        # from the API response if they were not provided by the user
        self._max_idle_time = self._seconds_to_timedelta(
            body.get("max_idle_time"))
        self._idle_seconds = self._seconds_to_timedelta(
            body.get("idle_seconds"))
        self._auto_terminate_ts = self._iso_to_datetime(
            body.get("auto_terminate_ts"))
        self._log_machine_group_info()

    @abstractmethod
    def __repr__(self):
        pass

    def active_machines_to_str(self) -> str:
        """Return the number of machines currently running.
        """
        return f"{self._active_machines}/{self.num_machines}"

    @classmethod
    def from_api_response(cls, resp: dict):
        """Creates a MachineGroup object from an API response."""

        machine_group = cls(
            machine_type=resp["machine_type"],
            data_disk_gb=resp["disk_size_gb"],
            register=False,
        )
        machine_group._id = resp["id"]
        machine_group.provider = resp["provider_id"]
        machine_group._name = resp["name"]
        machine_group.create_time = resp["creation_timestamp"]
        machine_group._started = bool(resp["started"])
        machine_group.quota_usage = resp.get("quota_usage") or {}
        machine_group._max_idle_time = cls._seconds_to_timedelta(
            resp.get("max_idle_time"))
        machine_group._idle_seconds = cls._seconds_to_timedelta(
            resp.get("idle_seconds"))
        machine_group._auto_terminate_ts = cls._iso_to_datetime(
            resp.get("auto_terminate_ts"))

        return machine_group

    def start(self, **kwargs):
        """Starts a machine group.

        Args:
            **kwargs: Depending on the type of machine group to be started,
              this can be num_machines, max_machines, min_machines,
              and is_elastic."""
        if self._started:
            logging.info("Attempting to start a machine group already started.")
            return

        if self.id is None or self.name is None:
            logging.info("Attempting to start an unregistered machine group. "
                         "Make sure you have called the constructor without "
                         "`register=False`.")
            return

        request_body = \
            inductiva.client.models.VMGroupConfig(
                id=self.id,
                name=self.name,
                machine_type=self.machine_type,
                provider_id=self.provider,
                threads_per_core=self.threads_per_core,
                disk_size_gb=self.data_disk_gb,
                **kwargs,
            )

        logging.info("Starting %s. "
                     "This may take a few minutes.", repr(self))
        logging.info("Note that stopping this local process will not interrupt "
                     "the creation of the machine group. Please wait...")
        start_time = time.time()
        self._api.start_vm_group(body=request_body)
        creation_time = format_utils.seconds_formatter(time.time() - start_time)
        self._started = True
        logging.info("%s successfully started in %s.", self, creation_time)
        logging.info("")
        logging.info("The machine group is using the following quotas:")
        self.log_quota_usage("used by resource")
        return True

    def terminate(self, verbose: bool = True, **kwargs):
        """Terminates a machine group."""
        if not self._started or self.id is None or self.name is None:
            logging.warning(
                "Attempting to terminate an unstarted machine group.")
            return

        try:
            request_body = \
                inductiva.client.models.VMGroupConfig(
                    id=self.id,
                    name=self.name,
                    machine_type=self.machine_type,
                    provider_id=self.provider,
                    threads_per_core=self.threads_per_core,
                    disk_size_gb=self.data_disk_gb,
                    **kwargs,
                )

            self._api.delete_vm_group(body=request_body)
            if verbose:
                logging.info("Successfully requested termination of %s.",
                             repr(self))
                logging.info("Termination of the machine group "
                             "freed the following quotas:")
                self.log_quota_usage("freed by resource")
            return True

        except inductiva.client.ApiException as api_exception:
            raise api_exception

    def _get_estimated_cost(self, spot: bool = True) -> float:
        """Returns estimate cost of a single machine in the group.

        This method is an overlay of the more general method, but
        it verifies if the cost has already been estimated and returns
        it immediately if it has.
        """
        if self.provider in (
                machine_types.ProviderType.ICE,
                machine_types.ProviderType.LOCAL,
        ):
            return 0

        self._estimated_cost = inductiva.resources.estimate_machine_cost(
            self.machine_type,
            spot,
        )

        return self._estimated_cost

    def log_quota_usage(self, resource_usage_header: str):
        quotas = users.get_quotas()
        table = defaultdict(list)
        emph_formatter = format_utils.get_ansi_formatter()

        header_formatters = [
            lambda x: emph_formatter(x.upper(), format_utils.Emphasis.BOLD)
        ]

        for name, value in self.quota_usage.items():
            max_allowed = quotas.get(name, {}).get("max_allowed", "n/a")
            full_name = quotas.get(name, {}).get("label", "n/a")
            in_use = quotas.get(name, {}).get("in_use", "n/a")

            table[""].append(full_name)
            table[resource_usage_header].append(value)
            table["current usage"].append(in_use)
            table["max allowed"].append(max_allowed)

        table_str = format_utils.get_tabular_str(
            table,
            header_formatters=header_formatters,
        )

        logging.info(table_str)

    def _log_estimated_spot_vm_savings(self) -> None:
        if self.provider in (
                machine_types.ProviderType.ICE,
                machine_types.ProviderType.LOCAL,
        ):
            return

        spot_cost = self._get_estimated_cost(True)
        non_spot_cost = self._get_estimated_cost(False)
        spot_times_cheaper = round(non_spot_cost / spot_cost, 2)

        is_spot = getattr(self, "spot", True)

        if not is_spot:
            logging.info(
                "\t· The same machine group with spot machines would cost "
                "%.1fx less. Specify "
                "`spot=True` in the constructor to use spot machines.",
                spot_times_cheaper,
            )
        else:
            logging.info(
                "\t· You are spending %.1fx less "
                "by using spot machines.",
                spot_times_cheaper,
            )
        logging.info("")

    def status(self):
        """Returns the status of a machine group if it exists.

        Otherwise returns None"""
        if self.name is None:
            logging.info(
                "Attempting to get the status of an unregistered machine group."
            )
            return

        response = self._api.get_group_status({"name": self.name})
        if response.body == "notFound":
            logging.info("Machine group does not exist: %s.", self.name)
        return response.body

    def _log_machine_group_info(self):
        """Logs the machine group info."""

        logging.info("\t· Name:                       %s", self.name)
        logging.info("\t· Machine Type:               %s", self.machine_type)
        logging.info("\t· Data disk size:             %s GB", self.data_disk_gb)

        # Log max idle time
        value_str = format_utils.timedelta_formatter(
            self.max_idle_time) if self.max_idle_time is not None else "N/A"
        logging.info("\t· Maximum idle time:          %s", value_str)

        # Log auto terminate timestamp
        value_str = self.auto_terminate_ts.strftime(
            "%Y/%m/%d %H:%M:%S"
        ) if self.auto_terminate_ts is not None else "N/A"
        logging.info("\t· Auto terminate timestamp:   %s", value_str)



"""OpenFAST module of the API for wind turbine simulations."""
from typing import Optional

from inductiva import types, tasks, simulators


class OpenFAST(simulators.Simulator):
    """Class to invoke a generic OpenFAST simulation on the API.

    """

    def __init__(self, /, version: Optional[str] = None, use_dev: bool = False):
        """Initialize the OpenFAST simulator.
        
        Args:
            version (str): The version of the simulator to use. If None, the
                latest available version in the platform is used.
            use_dev (bool): Request use of the development version of
                the simulator. By default (False), the production version
                is used.
        """
        super().__init__(version=version, use_dev=use_dev)
        self.api_method_name = "openfast.openfast.run_simulation"

    def run(self,
            input_dir: str,
            commands: types.Commands,
            on: Optional[types.ComputationalResources] = None,
            storage_dir: Optional[str] = "",
            extra_metadata: Optional[dict] = None,
            resubmit_on_preemption: bool = False,
            **kwargs) -> tasks.Task:
        """Run the simulation.

        Args:
            input_dir: Path to the directory of the simulation input files.
            commands: List of commands to run using the OpenFAST simulator.
            on: The computational resource to launch the simulation on. If None
                the simulation is submitted to a machine in the default pool.
            other arguments: See the documentation of the base class.
            resubmit_on_preemption (bool): Resubmit task for execution when
                previous execution attempts were preempted. Only applicable when
                using a preemptible resource, i.e., resource instantiates with
                `spot=True`.
        """
        return super().run(input_dir,
                           on=on,
                           commands=commands,
                           storage_dir=storage_dir,
                           extra_metadata=extra_metadata,
                           resubmit_on_preemption=resubmit_on_preemption,
                           **kwargs)



"""GROMACS module of the API"""

from typing import Optional

from inductiva import types, tasks, simulators


class GROMACS(simulators.Simulator):
    """Class to invoke any GROMACS command on the API."""

    def __init__(self, /, version: Optional[str] = None, use_dev: bool = False):
        """Initialize the GROMACS simulator.
        
        Args:
            version (str): The version of the simulator to use. If None, the
                latest available version in the platform is used.
            use_dev (bool): Request use of the development version of
                the simulator. By default (False), the production version
                is used.
        """
        super().__init__(version=version, use_dev=use_dev)
        self.api_method_name = "md.gromacs.run_simulation"

    def run(
        self,
        input_dir: str,
        commands: types.Commands,
        on: Optional[types.ComputationalResources] = None,
        storage_dir: Optional[str] = "",
        resubmit_on_preemption: bool = False,
        extra_metadata: Optional[dict] = None,
        **kwargs,
    ) -> tasks.Task:
        """Run a list of GROMACS commands.

        Args:
            input_dir: Path to the directory containing the input files.
            commands: List of commands to run using the GROMACS simulator.
            on: The computational resource to launch the simulation on. If None
                the simulation is submitted to a machine in the default pool.
            storage_dir: Parent directory for storing simulation
                               results.
            resubmit_on_preemption (bool): Resubmit task for execution when
                previous execution attempts were preempted. Only applicable when
                using a preemptible resource, i.e., resource instantiates with
                `spot=True`.
        """

        return super().run(input_dir,
                           on=on,
                           commands=commands,
                           storage_dir=storage_dir,
                           extra_metadata=extra_metadata,
                           resubmit_on_preemption=resubmit_on_preemption,
                           **kwargs)



"""Methods to interact with the available simulators."""
import requests
from collections import defaultdict
import re

# Regex to extract metadata from the image name
# Example:
#   dualsphysics_v5.2.1_dev -> name=dualsphysics, version=5.2.1, isdev=_dev
#   dualsphysics_v5.2.1 -> name=dualsphysics, version=5.2.1, isdev=None
IMAGE_TOKENIZER = re.compile(r"(?P<name>[a-zA-Z-_0-9]+)"
                             r"_v(?P<version>.+?(?=(_dev|$)))"
                             r"(?P<isdev>_dev)?")

DOCKERHUB_URL = "https://hub.docker.com/v2/repositories/inductiva/kutu/tags/"

SKIP_IMAGES = ["base-image", "echo"]


def list_available_images():
    """List available images on DockerHub for the Inductiva API.

    Returns:
        A dictionary with the available images for the API, grouped by branch
        (dev/prod) and simulator name.

    Example:
    >>> list_available_images()
    {
        {'dev':
            {'amr-wind': ['1.4.0'],
              ...
             'dualsphysics': ['5.2.1']
            },
        'prod':
            {'amr-wind': ['1.3.0', '1.4.0'],
              ...
             'dualsphysics': ['5.2.1']
            }
        }
    """

    images = defaultdict(lambda: defaultdict(list))
    url = f"{DOCKERHUB_URL}?page_size=100"

    while url is not None:
        resp = requests.get(url, timeout=10)
        data = resp.json()

        for tag in data["results"]:
            if (match := IMAGE_TOKENIZER.match(tag["name"])) is None:
                continue

            if match.group("name") in SKIP_IMAGES:
                continue

            name, version = match.group("name"), match.group("version")
            branch = "development" if match.group("isdev") else "production"
            images[branch][name].append(version)

        url = data["next"]

    return images



"""FEniCSx module of the API for Finite Element Analysis."""

from typing import Optional

from inductiva import simulators, types, tasks


class FEniCSx(simulators.Simulator):
    """Class to invoke a generic FEniCSx simulation on the API."""

    def __init__(self, /, version: Optional[str] = None, use_dev: bool = False):
        """Initialize the FEniCSx simulator.

        Args:
            version (str): The version of the simulator to use. If None, the
                latest available version in the platform is used.
            use_dev (bool): Request use of the development version of
                the simulator. By default (False), the production version
                is used.
        """
        super().__init__(version=version, use_dev=use_dev)
        self.api_method_name = "fem.fenicsx.run_simulation"

    def _get_image_uri(self):
        return None

    def run(self,
            input_dir: str,
            geometry_filename: str,
            bcs_filename: str,
            material_filename: str,
            mesh_filename: Optional[str] = None,
            mesh_info_filename: Optional[str] = None,
            on: Optional[types.ComputationalResources] = None,
            storage_dir: Optional[str] = "",
            extra_metadata: Optional[dict] = None,
            **kwargs) -> tasks.Task:
        """Run the simulation.

        Args:
            geometry_filename: Geometry filename.
            bcs_filename: Boundary conditions filename.
            material_filename: Material filename.
            mesh_filename: Mesh filename.
            mesh_info_filename: Mesh information filename.
            on: The computational resource to launch the simulation on. If None
                the simulation is submitted to a machine in the default pool.
            storage_dir: Parent directory for storing simulation results.
            **kwargs: Arbitrary keyword arguments, including:
                - global_refinement_meshing_factor (float): Factor for global
                  mesh refinement.
                - local_refinement_meshing_factor (float): Factor for local mesh
                  refinement.
                - smoothing_parameter (float): Mesh smoothing parameter.
                - mesh_element_family (str): Mesh element family.
                - mesh_element_order (int): Order of the mesh element.
                - mesh_quadrature_rule (str): Mesh quadrature rule.
                - mesh_quadrature_degree (int): Mesh quadrature degree.
            other arguments: See the documentation of the base class.
        """

        return super().run(input_dir,
                           mesh_info_filename=mesh_info_filename,
                           material_filename=material_filename,
                           geometry_filename=geometry_filename,
                           extra_metadata=extra_metadata,
                           mesh_filename=mesh_filename,
                           bcs_filename=bcs_filename,
                           storage_dir=storage_dir,
                           on=on,
                           **kwargs)



"""Localization module that provides support for multiple languages."""
from typing import Optional
from threading import RLock
import pathlib
import logging
import locale
import json

_INDUCTIVA_DEFAULT_LANG = "en"

_logger = logging.getLogger(__name__)


class Template:
    """Template for lazy string formatting.

    Formatting is performed using the str.format method only when a string
    representation of the object is required (typically using str(obj)).
    """

    def __init__(self, fmt: str, /, *args, **kwargs):
        """Intialize a Template object using the given formatting string
        and the positional and keyword substituion arguments.

        Args:
            fmt (str): The template string.
            *args: Positional arguments for the str.format method.
            **kwargs: Keyword arguments for the str.format method.
        """
        self.fmt = fmt
        self.args = args
        self.kwargs = kwargs

    def __str__(self) -> str:
        """Return a formatted version of self.fmt, using substitutions
        from self.args and self.kwargs."""
        return self.fmt.format(*self.args, **self.kwargs)


class Translator:
    """Translation provider with string rendering support.

    Attributes:
        TRANSLATIONS_DIR (pathlib.Path): Path to the directory where the
            translation files are stored.
    """

    TRANSLATIONS_DIR = pathname = pathlib.Path(__file__).parent / "translations"

    def __init__(self, lang: Optional[str] = None) -> None:
        self._translations = {}
        self._lock = RLock()
        self._lang = None

        # make sure the default language is loaded
        # before setting it to the required value
        self.set_lang(_INDUCTIVA_DEFAULT_LANG)
        self.set_lang(lang)

    def get_lang(self):
        """Get the current language."""
        return self._lang

    def set_lang(self, lang: str):
        """Set the current language.

        Args:
            lang (str): Name of the language to set.
        """
        with self._lock:
            # if None is provided, use the system's locale language if
            # available. Otherwise use the packages's default.
            if lang is None:
                loc, _ = locale.getlocale()
                lang = loc.split("_")[0] if loc else _INDUCTIVA_DEFAULT_LANG

            if lang in self._translations:
                return

            if self._load(lang):
                self._lang = lang

    def _load(self, lang: str) -> bool:
        """Load the translation file for the given language.

        Returns True if the language file was successfully
        loaded, False otherwise.
        NOTE: This method is not thread-safe.

        Args:
            lang (str): Name of the language to load.
        """
        if not lang:
            return False

        lang = lang.lower()
        pathname = self.TRANSLATIONS_DIR / f"{lang}.json"

        if not pathname.exists():
            # if translation file not available, there is not point in trying
            # to use a translation for this message. Just use the default.
            _logger.debug(
                "Translation file not available for language '%s'. "
                "Will use '%s' as default language.", lang,
                _INDUCTIVA_DEFAULT_LANG)
            return False

        with open(pathname, "r", encoding="utf-8") as f:
            translations = json.load(f)

        for key, translation in translations.items():
            if isinstance(translation, list):
                translations[key] = "\n".join(translation)

        self._translations[lang] = translations
        return True

    def __call__(self, key, *args, **kwargs) -> Template:
        """Return a Template object with the translation for the given key as
        formatting string, and substitutions from args and kwargs.

        The translation is retrieved from the active language. If the key is
        not found in the active language, the default language will be used.
        If the key doesn't exist in the default language, the key itself will
        be used as the translation.

        Args:
            key (str): Key for the translation which will be used as formatting
                string in the Template constructor.
            *args: Positional substitutions for the Template constructor.
            **kwargs: Keyword substitutions for the Template constructor.
        """
        translation = self[key]
        return Template(translation, *args, **kwargs)

    def __getitem__(self, key: str) -> str:
        """Get the translation for the given key in the active language.

        If the key is not found in the active language, the default language
        will be used. If the key doesn't exist in the default language, the key
        itself will be returned.

        Args:
            key (str): Key for the translation to get.
        """
        translations = self._translations.get(self._lang)
        if translations is None or key not in translations:
            translations = self._translations.get(_INDUCTIVA_DEFAULT_LANG)
        return translations.get(key, key)


# pseudo-singleton object that is already initialized to the system's locale
# or the package's default language if the system's locale is not available.
translator = Translator()



# coding: utf-8

"""
    InductivaWebAPI

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""

from datetime import date, datetime  # noqa: F401
import decimal  # noqa: F401
import functools  # noqa: F401
import io  # noqa: F401
import re  # noqa: F401
import typing  # noqa: F401
import typing_extensions  # noqa: F401
import uuid  # noqa: F401

import frozendict  # noqa: F401

from inductiva.client import schemas  # noqa: F401


class BaseVMGroup(
    schemas.DictSchema
):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.

    Machine group base schema.
    """


    class MetaOapg:
        
        class properties:
            
            
            class max_idle_time(
                schemas.ComposedSchema,
            ):
            
            
                class MetaOapg:
                    format = 'time-delta'
                    any_of_0 = schemas.NumberSchema
                    any_of_1 = schemas.NoneSchema
                    
                    @classmethod
                    @functools.lru_cache()
                    def any_of(cls):
                        # we need this here to make our import statements work
                        # we must store _composed_schemas in here so the code is only run
                        # when we invoke this method. If we kept this at the class
                        # level we would get an error because the class level
                        # code would be run when this module is imported, and these composed
                        # classes don't exist yet because their module has not finished
                        # loading
                        return [
                            cls.any_of_0,
                            cls.any_of_1,
                        ]
            
            
                def __new__(
                    cls,
                    *_args: typing.Union[dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, bool, None, list, tuple, bytes, io.FileIO, io.BufferedReader, ],
                    _configuration: typing.Optional[schemas.Configuration] = None,
                    **kwargs: typing.Union[schemas.AnyTypeSchema, dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, None, list, tuple, bytes],
                ) -> 'max_idle_time':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )
            
            
            class auto_terminate_ts(
                schemas.DateTimeBase,
                schemas.ComposedSchema,
            ):
            
            
                class MetaOapg:
                    format = 'date-time'
                    any_of_0 = schemas.StrSchema
                    any_of_1 = schemas.NoneSchema
                    
                    @classmethod
                    @functools.lru_cache()
                    def any_of(cls):
                        # we need this here to make our import statements work
                        # we must store _composed_schemas in here so the code is only run
                        # when we invoke this method. If we kept this at the class
                        # level we would get an error because the class level
                        # code would be run when this module is imported, and these composed
                        # classes don't exist yet because their module has not finished
                        # loading
                        return [
                            cls.any_of_0,
                            cls.any_of_1,
                        ]
            
            
                def __new__(
                    cls,
                    *_args: typing.Union[dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, bool, None, list, tuple, bytes, io.FileIO, io.BufferedReader, ],
                    _configuration: typing.Optional[schemas.Configuration] = None,
                    **kwargs: typing.Union[schemas.AnyTypeSchema, dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, None, list, tuple, bytes],
                ) -> 'auto_terminate_ts':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )
            
            
            class id(
                schemas.UUIDBase,
                schemas.ComposedSchema,
            ):
            
            
                class MetaOapg:
                    format = 'uuid'
                    any_of_0 = schemas.StrSchema
                    any_of_1 = schemas.NoneSchema
                    
                    @classmethod
                    @functools.lru_cache()
                    def any_of(cls):
                        # we need this here to make our import statements work
                        # we must store _composed_schemas in here so the code is only run
                        # when we invoke this method. If we kept this at the class
                        # level we would get an error because the class level
                        # code would be run when this module is imported, and these composed
                        # classes don't exist yet because their module has not finished
                        # loading
                        return [
                            cls.any_of_0,
                            cls.any_of_1,
                        ]
            
            
                def __new__(
                    cls,
                    *_args: typing.Union[dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, bool, None, list, tuple, bytes, io.FileIO, io.BufferedReader, ],
                    _configuration: typing.Optional[schemas.Configuration] = None,
                    **kwargs: typing.Union[schemas.AnyTypeSchema, dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, None, list, tuple, bytes],
                ) -> 'id':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )
            
            
            class name(
                schemas.ComposedSchema,
            ):
            
            
                class MetaOapg:
                    any_of_0 = schemas.StrSchema
                    any_of_1 = schemas.NoneSchema
                    
                    @classmethod
                    @functools.lru_cache()
                    def any_of(cls):
                        # we need this here to make our import statements work
                        # we must store _composed_schemas in here so the code is only run
                        # when we invoke this method. If we kept this at the class
                        # level we would get an error because the class level
                        # code would be run when this module is imported, and these composed
                        # classes don't exist yet because their module has not finished
                        # loading
                        return [
                            cls.any_of_0,
                            cls.any_of_1,
                        ]
            
            
                def __new__(
                    cls,
                    *_args: typing.Union[dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, bool, None, list, tuple, bytes, io.FileIO, io.BufferedReader, ],
                    _configuration: typing.Optional[schemas.Configuration] = None,
                    **kwargs: typing.Union[schemas.AnyTypeSchema, dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, None, list, tuple, bytes],
                ) -> 'name':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )
            
            
            class machine_type(
                schemas.ComposedSchema,
            ):
            
            
                class MetaOapg:
                    any_of_0 = schemas.StrSchema
                    any_of_1 = schemas.NoneSchema
                    
                    @classmethod
                    @functools.lru_cache()
                    def any_of(cls):
                        # we need this here to make our import statements work
                        # we must store _composed_schemas in here so the code is only run
                        # when we invoke this method. If we kept this at the class
                        # level we would get an error because the class level
                        # code would be run when this module is imported, and these composed
                        # classes don't exist yet because their module has not finished
                        # loading
                        return [
                            cls.any_of_0,
                            cls.any_of_1,
                        ]
            
            
                def __new__(
                    cls,
                    *_args: typing.Union[dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, bool, None, list, tuple, bytes, io.FileIO, io.BufferedReader, ],
                    _configuration: typing.Optional[schemas.Configuration] = None,
                    **kwargs: typing.Union[schemas.AnyTypeSchema, dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, None, list, tuple, bytes],
                ) -> 'machine_type':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )
            
            
            class disk_size_gb(
                schemas.ComposedSchema,
            ):
            
            
                class MetaOapg:
                    any_of_0 = schemas.IntSchema
                    any_of_1 = schemas.NoneSchema
                    
                    @classmethod
                    @functools.lru_cache()
                    def any_of(cls):
                        # we need this here to make our import statements work
                        # we must store _composed_schemas in here so the code is only run
                        # when we invoke this method. If we kept this at the class
                        # level we would get an error because the class level
                        # code would be run when this module is imported, and these composed
                        # classes don't exist yet because their module has not finished
                        # loading
                        return [
                            cls.any_of_0,
                            cls.any_of_1,
                        ]
            
            
                def __new__(
                    cls,
                    *_args: typing.Union[dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, bool, None, list, tuple, bytes, io.FileIO, io.BufferedReader, ],
                    _configuration: typing.Optional[schemas.Configuration] = None,
                    **kwargs: typing.Union[schemas.AnyTypeSchema, dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, None, list, tuple, bytes],
                ) -> 'disk_size_gb':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )
            
            
            class threads_per_core(
                schemas.ComposedSchema,
            ):
            
            
                class MetaOapg:
                    any_of_0 = schemas.IntSchema
                    any_of_1 = schemas.NoneSchema
                    
                    @classmethod
                    @functools.lru_cache()
                    def any_of(cls):
                        # we need this here to make our import statements work
                        # we must store _composed_schemas in here so the code is only run
                        # when we invoke this method. If we kept this at the class
                        # level we would get an error because the class level
                        # code would be run when this module is imported, and these composed
                        # classes don't exist yet because their module has not finished
                        # loading
                        return [
                            cls.any_of_0,
                            cls.any_of_1,
                        ]
            
            
                def __new__(
                    cls,
                    *_args: typing.Union[dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, bool, None, list, tuple, bytes, io.FileIO, io.BufferedReader, ],
                    _configuration: typing.Optional[schemas.Configuration] = None,
                    **kwargs: typing.Union[schemas.AnyTypeSchema, dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, None, list, tuple, bytes],
                ) -> 'threads_per_core':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )
            
            
            class num_vms(
                schemas.ComposedSchema,
            ):
            
            
                class MetaOapg:
                    any_of_0 = schemas.IntSchema
                    any_of_1 = schemas.NoneSchema
                    
                    @classmethod
                    @functools.lru_cache()
                    def any_of(cls):
                        # we need this here to make our import statements work
                        # we must store _composed_schemas in here so the code is only run
                        # when we invoke this method. If we kept this at the class
                        # level we would get an error because the class level
                        # code would be run when this module is imported, and these composed
                        # classes don't exist yet because their module has not finished
                        # loading
                        return [
                            cls.any_of_0,
                            cls.any_of_1,
                        ]
            
            
                def __new__(
                    cls,
                    *_args: typing.Union[dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, bool, None, list, tuple, bytes, io.FileIO, io.BufferedReader, ],
                    _configuration: typing.Optional[schemas.Configuration] = None,
                    **kwargs: typing.Union[schemas.AnyTypeSchema, dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, None, list, tuple, bytes],
                ) -> 'num_vms':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )
            
            
            class idle_seconds(
                schemas.ComposedSchema,
            ):
            
            
                class MetaOapg:
                    any_of_0 = schemas.IntSchema
                    any_of_1 = schemas.NoneSchema
                    
                    @classmethod
                    @functools.lru_cache()
                    def any_of(cls):
                        # we need this here to make our import statements work
                        # we must store _composed_schemas in here so the code is only run
                        # when we invoke this method. If we kept this at the class
                        # level we would get an error because the class level
                        # code would be run when this module is imported, and these composed
                        # classes don't exist yet because their module has not finished
                        # loading
                        return [
                            cls.any_of_0,
                            cls.any_of_1,
                        ]
            
            
                def __new__(
                    cls,
                    *_args: typing.Union[dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, bool, None, list, tuple, bytes, io.FileIO, io.BufferedReader, ],
                    _configuration: typing.Optional[schemas.Configuration] = None,
                    **kwargs: typing.Union[schemas.AnyTypeSchema, dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, None, list, tuple, bytes],
                ) -> 'idle_seconds':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )
            
            
            class creation_timestamp(
                schemas.DateTimeBase,
                schemas.ComposedSchema,
            ):
            
            
                class MetaOapg:
                    format = 'date-time'
                    any_of_0 = schemas.StrSchema
                    any_of_1 = schemas.NoneSchema
                    
                    @classmethod
                    @functools.lru_cache()
                    def any_of(cls):
                        # we need this here to make our import statements work
                        # we must store _composed_schemas in here so the code is only run
                        # when we invoke this method. If we kept this at the class
                        # level we would get an error because the class level
                        # code would be run when this module is imported, and these composed
                        # classes don't exist yet because their module has not finished
                        # loading
                        return [
                            cls.any_of_0,
                            cls.any_of_1,
                        ]
            
            
                def __new__(
                    cls,
                    *_args: typing.Union[dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, bool, None, list, tuple, bytes, io.FileIO, io.BufferedReader, ],
                    _configuration: typing.Optional[schemas.Configuration] = None,
                    **kwargs: typing.Union[schemas.AnyTypeSchema, dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, None, list, tuple, bytes],
                ) -> 'creation_timestamp':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )
            
            
            class deletion_timestamp(
                schemas.DateTimeBase,
                schemas.ComposedSchema,
            ):
            
            
                class MetaOapg:
                    format = 'date-time'
                    any_of_0 = schemas.StrSchema
                    any_of_1 = schemas.NoneSchema
                    
                    @classmethod
                    @functools.lru_cache()
                    def any_of(cls):
                        # we need this here to make our import statements work
                        # we must store _composed_schemas in here so the code is only run
                        # when we invoke this method. If we kept this at the class
                        # level we would get an error because the class level
                        # code would be run when this module is imported, and these composed
                        # classes don't exist yet because their module has not finished
                        # loading
                        return [
                            cls.any_of_0,
                            cls.any_of_1,
                        ]
            
            
                def __new__(
                    cls,
                    *_args: typing.Union[dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, bool, None, list, tuple, bytes, io.FileIO, io.BufferedReader, ],
                    _configuration: typing.Optional[schemas.Configuration] = None,
                    **kwargs: typing.Union[schemas.AnyTypeSchema, dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, None, list, tuple, bytes],
                ) -> 'deletion_timestamp':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )
            
            
            class type(
                schemas.ComposedSchema,
            ):
            
            
                class MetaOapg:
                    
                    @classmethod
                    @functools.lru_cache()
                    def all_of(cls):
                        # we need this here to make our import statements work
                        # we must store _composed_schemas in here so the code is only run
                        # when we invoke this method. If we kept this at the class
                        # level we would get an error because the class level
                        # code would be run when this module is imported, and these composed
                        # classes don't exist yet because their module has not finished
                        # loading
                        return [
                            MachineGroupType,
                        ]
            
            
                def __new__(
                    cls,
                    *_args: typing.Union[dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, bool, None, list, tuple, bytes, io.FileIO, io.BufferedReader, ],
                    _configuration: typing.Optional[schemas.Configuration] = None,
                    **kwargs: typing.Union[schemas.AnyTypeSchema, dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, None, list, tuple, bytes],
                ) -> 'type':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )
            
            
            class provider_id(
                schemas.ComposedSchema,
            ):
            
            
                class MetaOapg:
                    
                    @classmethod
                    @functools.lru_cache()
                    def all_of(cls):
                        # we need this here to make our import statements work
                        # we must store _composed_schemas in here so the code is only run
                        # when we invoke this method. If we kept this at the class
                        # level we would get an error because the class level
                        # code would be run when this module is imported, and these composed
                        # classes don't exist yet because their module has not finished
                        # loading
                        return [
                            Providers,
                        ]
            
            
                def __new__(
                    cls,
                    *_args: typing.Union[dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, bool, None, list, tuple, bytes, io.FileIO, io.BufferedReader, ],
                    _configuration: typing.Optional[schemas.Configuration] = None,
                    **kwargs: typing.Union[schemas.AnyTypeSchema, dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, None, list, tuple, bytes],
                ) -> 'provider_id':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )
            started = schemas.BoolSchema
            
            
            class quota_usage(
                schemas.ComposedSchema,
            ):
            
            
                class MetaOapg:
                    additional_properties = schemas.NumberSchema
                    any_of_0 = schemas.DictSchema
                    any_of_1 = schemas.NoneSchema
                    
                    @classmethod
                    @functools.lru_cache()
                    def any_of(cls):
                        # we need this here to make our import statements work
                        # we must store _composed_schemas in here so the code is only run
                        # when we invoke this method. If we kept this at the class
                        # level we would get an error because the class level
                        # code would be run when this module is imported, and these composed
                        # classes don't exist yet because their module has not finished
                        # loading
                        return [
                            cls.any_of_0,
                            cls.any_of_1,
                        ]
            
                
                def __getitem__(self, name: typing.Union[str, ]) -> MetaOapg.additional_properties:
                    # dict_instance[name] accessor
                    return super().__getitem__(name)
                
                def get_item_oapg(self, name: typing.Union[str, ]) -> MetaOapg.additional_properties:
                    return super().get_item_oapg(name)
            
                def __new__(
                    cls,
                    *_args: typing.Union[dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, bool, None, list, tuple, bytes, io.FileIO, io.BufferedReader, ],
                    _configuration: typing.Optional[schemas.Configuration] = None,
                    **kwargs: typing.Union[MetaOapg.additional_properties, decimal.Decimal, int, float, ],
                ) -> 'quota_usage':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )
            
            
            class status(
                schemas.ComposedSchema,
            ):
            
            
                class MetaOapg:
                    any_of_1 = schemas.NoneSchema
                    
                    @classmethod
                    @functools.lru_cache()
                    def any_of(cls):
                        # we need this here to make our import statements work
                        # we must store _composed_schemas in here so the code is only run
                        # when we invoke this method. If we kept this at the class
                        # level we would get an error because the class level
                        # code would be run when this module is imported, and these composed
                        # classes don't exist yet because their module has not finished
                        # loading
                        return [
                            MachineGroupStatus,
                            cls.any_of_1,
                        ]
            
            
                def __new__(
                    cls,
                    *_args: typing.Union[dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, bool, None, list, tuple, bytes, io.FileIO, io.BufferedReader, ],
                    _configuration: typing.Optional[schemas.Configuration] = None,
                    **kwargs: typing.Union[schemas.AnyTypeSchema, dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, None, list, tuple, bytes],
                ) -> 'status':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )
            __annotations__ = {
                "max_idle_time": max_idle_time,
                "auto_terminate_ts": auto_terminate_ts,
                "id": id,
                "name": name,
                "machine_type": machine_type,
                "disk_size_gb": disk_size_gb,
                "threads_per_core": threads_per_core,
                "num_vms": num_vms,
                "idle_seconds": idle_seconds,
                "creation_timestamp": creation_timestamp,
                "deletion_timestamp": deletion_timestamp,
                "type": type,
                "provider_id": provider_id,
                "started": started,
                "quota_usage": quota_usage,
                "status": status,
            }
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["max_idle_time"]) -> MetaOapg.properties.max_idle_time: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["auto_terminate_ts"]) -> MetaOapg.properties.auto_terminate_ts: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["id"]) -> MetaOapg.properties.id: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["name"]) -> MetaOapg.properties.name: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["machine_type"]) -> MetaOapg.properties.machine_type: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["disk_size_gb"]) -> MetaOapg.properties.disk_size_gb: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["threads_per_core"]) -> MetaOapg.properties.threads_per_core: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["num_vms"]) -> MetaOapg.properties.num_vms: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["idle_seconds"]) -> MetaOapg.properties.idle_seconds: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["creation_timestamp"]) -> MetaOapg.properties.creation_timestamp: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["deletion_timestamp"]) -> MetaOapg.properties.deletion_timestamp: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["type"]) -> MetaOapg.properties.type: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["provider_id"]) -> MetaOapg.properties.provider_id: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["started"]) -> MetaOapg.properties.started: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["quota_usage"]) -> MetaOapg.properties.quota_usage: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["status"]) -> MetaOapg.properties.status: ...
    
    @typing.overload
    def __getitem__(self, name: str) -> schemas.UnsetAnyTypeSchema: ...
    
    def __getitem__(self, name: typing.Union[typing_extensions.Literal["max_idle_time", "auto_terminate_ts", "id", "name", "machine_type", "disk_size_gb", "threads_per_core", "num_vms", "idle_seconds", "creation_timestamp", "deletion_timestamp", "type", "provider_id", "started", "quota_usage", "status", ], str]):
        # dict_instance[name] accessor
        return super().__getitem__(name)
    
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["max_idle_time"]) -> typing.Union[MetaOapg.properties.max_idle_time, schemas.Unset]: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["auto_terminate_ts"]) -> typing.Union[MetaOapg.properties.auto_terminate_ts, schemas.Unset]: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["id"]) -> typing.Union[MetaOapg.properties.id, schemas.Unset]: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["name"]) -> typing.Union[MetaOapg.properties.name, schemas.Unset]: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["machine_type"]) -> typing.Union[MetaOapg.properties.machine_type, schemas.Unset]: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["disk_size_gb"]) -> typing.Union[MetaOapg.properties.disk_size_gb, schemas.Unset]: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["threads_per_core"]) -> typing.Union[MetaOapg.properties.threads_per_core, schemas.Unset]: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["num_vms"]) -> typing.Union[MetaOapg.properties.num_vms, schemas.Unset]: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["idle_seconds"]) -> typing.Union[MetaOapg.properties.idle_seconds, schemas.Unset]: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["creation_timestamp"]) -> typing.Union[MetaOapg.properties.creation_timestamp, schemas.Unset]: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["deletion_timestamp"]) -> typing.Union[MetaOapg.properties.deletion_timestamp, schemas.Unset]: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["type"]) -> typing.Union[MetaOapg.properties.type, schemas.Unset]: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["provider_id"]) -> typing.Union[MetaOapg.properties.provider_id, schemas.Unset]: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["started"]) -> typing.Union[MetaOapg.properties.started, schemas.Unset]: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["quota_usage"]) -> typing.Union[MetaOapg.properties.quota_usage, schemas.Unset]: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["status"]) -> typing.Union[MetaOapg.properties.status, schemas.Unset]: ...
    
    @typing.overload
    def get_item_oapg(self, name: str) -> typing.Union[schemas.UnsetAnyTypeSchema, schemas.Unset]: ...
    
    def get_item_oapg(self, name: typing.Union[typing_extensions.Literal["max_idle_time", "auto_terminate_ts", "id", "name", "machine_type", "disk_size_gb", "threads_per_core", "num_vms", "idle_seconds", "creation_timestamp", "deletion_timestamp", "type", "provider_id", "started", "quota_usage", "status", ], str]):
        return super().get_item_oapg(name)
    

    def __new__(
        cls,
        *_args: typing.Union[dict, frozendict.frozendict, ],
        max_idle_time: typing.Union[MetaOapg.properties.max_idle_time, dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, bool, None, list, tuple, bytes, io.FileIO, io.BufferedReader, schemas.Unset] = schemas.unset,
        auto_terminate_ts: typing.Union[MetaOapg.properties.auto_terminate_ts, dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, bool, None, list, tuple, bytes, io.FileIO, io.BufferedReader, schemas.Unset] = schemas.unset,
        id: typing.Union[MetaOapg.properties.id, dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, bool, None, list, tuple, bytes, io.FileIO, io.BufferedReader, schemas.Unset] = schemas.unset,
        name: typing.Union[MetaOapg.properties.name, dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, bool, None, list, tuple, bytes, io.FileIO, io.BufferedReader, schemas.Unset] = schemas.unset,
        machine_type: typing.Union[MetaOapg.properties.machine_type, dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, bool, None, list, tuple, bytes, io.FileIO, io.BufferedReader, schemas.Unset] = schemas.unset,
        disk_size_gb: typing.Union[MetaOapg.properties.disk_size_gb, dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, bool, None, list, tuple, bytes, io.FileIO, io.BufferedReader, schemas.Unset] = schemas.unset,
        threads_per_core: typing.Union[MetaOapg.properties.threads_per_core, dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, bool, None, list, tuple, bytes, io.FileIO, io.BufferedReader, schemas.Unset] = schemas.unset,
        num_vms: typing.Union[MetaOapg.properties.num_vms, dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, bool, None, list, tuple, bytes, io.FileIO, io.BufferedReader, schemas.Unset] = schemas.unset,
        idle_seconds: typing.Union[MetaOapg.properties.idle_seconds, dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, bool, None, list, tuple, bytes, io.FileIO, io.BufferedReader, schemas.Unset] = schemas.unset,
        creation_timestamp: typing.Union[MetaOapg.properties.creation_timestamp, dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, bool, None, list, tuple, bytes, io.FileIO, io.BufferedReader, schemas.Unset] = schemas.unset,
        deletion_timestamp: typing.Union[MetaOapg.properties.deletion_timestamp, dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, bool, None, list, tuple, bytes, io.FileIO, io.BufferedReader, schemas.Unset] = schemas.unset,
        type: typing.Union[MetaOapg.properties.type, dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, bool, None, list, tuple, bytes, io.FileIO, io.BufferedReader, schemas.Unset] = schemas.unset,
        provider_id: typing.Union[MetaOapg.properties.provider_id, dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, bool, None, list, tuple, bytes, io.FileIO, io.BufferedReader, schemas.Unset] = schemas.unset,
        started: typing.Union[MetaOapg.properties.started, bool, schemas.Unset] = schemas.unset,
        quota_usage: typing.Union[MetaOapg.properties.quota_usage, dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, bool, None, list, tuple, bytes, io.FileIO, io.BufferedReader, schemas.Unset] = schemas.unset,
        status: typing.Union[MetaOapg.properties.status, dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, bool, None, list, tuple, bytes, io.FileIO, io.BufferedReader, schemas.Unset] = schemas.unset,
        _configuration: typing.Optional[schemas.Configuration] = None,
        **kwargs: typing.Union[schemas.AnyTypeSchema, dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, None, list, tuple, bytes],
    ) -> 'BaseVMGroup':
        return super().__new__(
            cls,
            *_args,
            max_idle_time=max_idle_time,
            auto_terminate_ts=auto_terminate_ts,
            id=id,
            name=name,
            machine_type=machine_type,
            disk_size_gb=disk_size_gb,
            threads_per_core=threads_per_core,
            num_vms=num_vms,
            idle_seconds=idle_seconds,
            creation_timestamp=creation_timestamp,
            deletion_timestamp=deletion_timestamp,
            type=type,
            provider_id=provider_id,
            started=started,
            quota_usage=quota_usage,
            status=status,
            _configuration=_configuration,
            **kwargs,
        )

from inductiva.client.model.machine_group_status import MachineGroupStatus
from inductiva.client.model.machine_group_type import MachineGroupType
from inductiva.client.model.providers import Providers



# coding: utf-8

"""
    InductivaWebAPI

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""

from datetime import date, datetime  # noqa: F401
import decimal  # noqa: F401
import functools  # noqa: F401
import io  # noqa: F401
import re  # noqa: F401
import typing  # noqa: F401
import typing_extensions  # noqa: F401
import uuid  # noqa: F401

import frozendict  # noqa: F401

from inductiva.client import schemas  # noqa: F401


class User(
    schemas.DictSchema
):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """


    class MetaOapg:
        required = {
            "terms_and_conditions_decision",
            "campaigns",
            "tier",
            "total_available_credits",
            "email",
            "username",
        }
        
        class properties:
            username = schemas.StrSchema
            email = schemas.StrSchema
        
            @staticmethod
            def terms_and_conditions_decision() -> typing.Type['TermsAndConditions']:
                return TermsAndConditions
            total_available_credits = schemas.NumberSchema
        
            @staticmethod
            def tier() -> typing.Type['Tier']:
                return Tier
            
            
            class campaigns(
                schemas.ListSchema
            ):
            
            
                class MetaOapg:
                    
                    @staticmethod
                    def items() -> typing.Type['UserCampaign']:
                        return UserCampaign
            
                def __new__(
                    cls,
                    _arg: typing.Union[typing.Tuple['UserCampaign'], typing.List['UserCampaign']],
                    _configuration: typing.Optional[schemas.Configuration] = None,
                ) -> 'campaigns':
                    return super().__new__(
                        cls,
                        _arg,
                        _configuration=_configuration,
                    )
            
                def __getitem__(self, i: int) -> 'UserCampaign':
                    return super().__getitem__(i)
            
            
            class name(
                schemas.ComposedSchema,
            ):
            
            
                class MetaOapg:
                    any_of_0 = schemas.StrSchema
                    any_of_1 = schemas.NoneSchema
                    
                    @classmethod
                    @functools.lru_cache()
                    def any_of(cls):
                        # we need this here to make our import statements work
                        # we must store _composed_schemas in here so the code is only run
                        # when we invoke this method. If we kept this at the class
                        # level we would get an error because the class level
                        # code would be run when this module is imported, and these composed
                        # classes don't exist yet because their module has not finished
                        # loading
                        return [
                            cls.any_of_0,
                            cls.any_of_1,
                        ]
            
            
                def __new__(
                    cls,
                    *_args: typing.Union[dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, bool, None, list, tuple, bytes, io.FileIO, io.BufferedReader, ],
                    _configuration: typing.Optional[schemas.Configuration] = None,
                    **kwargs: typing.Union[schemas.AnyTypeSchema, dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, None, list, tuple, bytes],
                ) -> 'name':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )
            
            
            class terms_and_conditions_decision_ts(
                schemas.DateTimeBase,
                schemas.ComposedSchema,
            ):
            
            
                class MetaOapg:
                    format = 'date-time'
                    any_of_0 = schemas.StrSchema
                    any_of_1 = schemas.NoneSchema
                    
                    @classmethod
                    @functools.lru_cache()
                    def any_of(cls):
                        # we need this here to make our import statements work
                        # we must store _composed_schemas in here so the code is only run
                        # when we invoke this method. If we kept this at the class
                        # level we would get an error because the class level
                        # code would be run when this module is imported, and these composed
                        # classes don't exist yet because their module has not finished
                        # loading
                        return [
                            cls.any_of_0,
                            cls.any_of_1,
                        ]
            
            
                def __new__(
                    cls,
                    *_args: typing.Union[dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, bool, None, list, tuple, bytes, io.FileIO, io.BufferedReader, ],
                    _configuration: typing.Optional[schemas.Configuration] = None,
                    **kwargs: typing.Union[schemas.AnyTypeSchema, dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, None, list, tuple, bytes],
                ) -> 'terms_and_conditions_decision_ts':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )
            credits_currency = schemas.StrSchema
            __annotations__ = {
                "username": username,
                "email": email,
                "terms_and_conditions_decision": terms_and_conditions_decision,
                "total_available_credits": total_available_credits,
                "tier": tier,
                "campaigns": campaigns,
                "name": name,
                "terms_and_conditions_decision_ts": terms_and_conditions_decision_ts,
                "credits_currency": credits_currency,
            }
    
    terms_and_conditions_decision: 'TermsAndConditions'
    campaigns: MetaOapg.properties.campaigns
    tier: 'Tier'
    total_available_credits: MetaOapg.properties.total_available_credits
    email: MetaOapg.properties.email
    username: MetaOapg.properties.username
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["username"]) -> MetaOapg.properties.username: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["email"]) -> MetaOapg.properties.email: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["terms_and_conditions_decision"]) -> 'TermsAndConditions': ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["total_available_credits"]) -> MetaOapg.properties.total_available_credits: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["tier"]) -> 'Tier': ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["campaigns"]) -> MetaOapg.properties.campaigns: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["name"]) -> MetaOapg.properties.name: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["terms_and_conditions_decision_ts"]) -> MetaOapg.properties.terms_and_conditions_decision_ts: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["credits_currency"]) -> MetaOapg.properties.credits_currency: ...
    
    @typing.overload
    def __getitem__(self, name: str) -> schemas.UnsetAnyTypeSchema: ...
    
    def __getitem__(self, name: typing.Union[typing_extensions.Literal["username", "email", "terms_and_conditions_decision", "total_available_credits", "tier", "campaigns", "name", "terms_and_conditions_decision_ts", "credits_currency", ], str]):
        # dict_instance[name] accessor
        return super().__getitem__(name)
    
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["username"]) -> MetaOapg.properties.username: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["email"]) -> MetaOapg.properties.email: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["terms_and_conditions_decision"]) -> 'TermsAndConditions': ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["total_available_credits"]) -> MetaOapg.properties.total_available_credits: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["tier"]) -> 'Tier': ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["campaigns"]) -> MetaOapg.properties.campaigns: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["name"]) -> typing.Union[MetaOapg.properties.name, schemas.Unset]: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["terms_and_conditions_decision_ts"]) -> typing.Union[MetaOapg.properties.terms_and_conditions_decision_ts, schemas.Unset]: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["credits_currency"]) -> typing.Union[MetaOapg.properties.credits_currency, schemas.Unset]: ...
    
    @typing.overload
    def get_item_oapg(self, name: str) -> typing.Union[schemas.UnsetAnyTypeSchema, schemas.Unset]: ...
    
    def get_item_oapg(self, name: typing.Union[typing_extensions.Literal["username", "email", "terms_and_conditions_decision", "total_available_credits", "tier", "campaigns", "name", "terms_and_conditions_decision_ts", "credits_currency", ], str]):
        return super().get_item_oapg(name)
    

    def __new__(
        cls,
        *_args: typing.Union[dict, frozendict.frozendict, ],
        terms_and_conditions_decision: 'TermsAndConditions',
        campaigns: typing.Union[MetaOapg.properties.campaigns, list, tuple, ],
        tier: 'Tier',
        total_available_credits: typing.Union[MetaOapg.properties.total_available_credits, decimal.Decimal, int, float, ],
        email: typing.Union[MetaOapg.properties.email, str, ],
        username: typing.Union[MetaOapg.properties.username, str, ],
        name: typing.Union[MetaOapg.properties.name, dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, bool, None, list, tuple, bytes, io.FileIO, io.BufferedReader, schemas.Unset] = schemas.unset,
        terms_and_conditions_decision_ts: typing.Union[MetaOapg.properties.terms_and_conditions_decision_ts, dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, bool, None, list, tuple, bytes, io.FileIO, io.BufferedReader, schemas.Unset] = schemas.unset,
        credits_currency: typing.Union[MetaOapg.properties.credits_currency, str, schemas.Unset] = schemas.unset,
        _configuration: typing.Optional[schemas.Configuration] = None,
        **kwargs: typing.Union[schemas.AnyTypeSchema, dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, None, list, tuple, bytes],
    ) -> 'User':
        return super().__new__(
            cls,
            *_args,
            terms_and_conditions_decision=terms_and_conditions_decision,
            campaigns=campaigns,
            tier=tier,
            total_available_credits=total_available_credits,
            email=email,
            username=username,
            name=name,
            terms_and_conditions_decision_ts=terms_and_conditions_decision_ts,
            credits_currency=credits_currency,
            _configuration=_configuration,
            **kwargs,
        )

from inductiva.client.model.terms_and_conditions import TermsAndConditions
from inductiva.client.model.tier import Tier
from inductiva.client.model.user_campaign import UserCampaign



# coding: utf-8
"""
    InductivaWebAPI

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""

from datetime import date, datetime  # noqa: F401
import decimal  # noqa: F401
import functools  # noqa: F401
import io  # noqa: F401
import re  # noqa: F401
import typing  # noqa: F401
import typing_extensions  # noqa: F401
import uuid  # noqa: F401

import frozendict  # noqa: F401

from inductiva.client import schemas  # noqa: F401


class Quota(schemas.DictSchema):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    class MetaOapg:
        required = {
            "unit",
            "scope",
            "id",
            "label",
        }

        class properties:
            id = schemas.StrSchema
            label = schemas.StrSchema
            unit = schemas.StrSchema

            @staticmethod
            def scope() -> typing.Type['QuotaScope']:
                return QuotaScope

            class max_allowed(
                    schemas.ComposedSchema,):

                class MetaOapg:
                    any_of_0 = schemas.NumberSchema
                    any_of_1 = schemas.NoneSchema

                    @classmethod
                    @functools.lru_cache()
                    def any_of(cls):
                        # we need this here to make our import statements work
                        # we must store _composed_schemas in here so the code is only run
                        # when we invoke this method. If we kept this at the class
                        # level we would get an error because the class level
                        # code would be run when this module is imported, and these composed
                        # classes don't exist yet because their module has not finished
                        # loading
                        return [
                            cls.any_of_0,
                            cls.any_of_1,
                        ]

                def __new__(
                    cls,
                    *_args: typing.Union[
                        dict,
                        frozendict.frozendict,
                        str,
                        date,
                        datetime,
                        uuid.UUID,
                        int,
                        float,
                        decimal.Decimal,
                        bool,
                        None,
                        list,
                        tuple,
                        bytes,
                        io.FileIO,
                        io.BufferedReader,
                    ],
                    _configuration: typing.Optional[
                        schemas.Configuration] = None,
                    **kwargs: typing.Union[schemas.AnyTypeSchema, dict,
                                           frozendict.frozendict, str, date,
                                           datetime, uuid.UUID, int, float,
                                           decimal.Decimal, None, list, tuple,
                                           bytes],
                ) -> 'max_allowed':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )

            class in_use(
                    schemas.ComposedSchema,):

                class MetaOapg:
                    any_of_0 = schemas.NumberSchema
                    any_of_1 = schemas.NoneSchema

                    @classmethod
                    @functools.lru_cache()
                    def any_of(cls):
                        # we need this here to make our import statements work
                        # we must store _composed_schemas in here so the code is only run
                        # when we invoke this method. If we kept this at the class
                        # level we would get an error because the class level
                        # code would be run when this module is imported, and these composed
                        # classes don't exist yet because their module has not finished
                        # loading
                        return [
                            cls.any_of_0,
                            cls.any_of_1,
                        ]

                def __new__(
                    cls,
                    *_args: typing.Union[
                        dict,
                        frozendict.frozendict,
                        str,
                        date,
                        datetime,
                        uuid.UUID,
                        int,
                        float,
                        decimal.Decimal,
                        bool,
                        None,
                        list,
                        tuple,
                        bytes,
                        io.FileIO,
                        io.BufferedReader,
                    ],
                    _configuration: typing.Optional[
                        schemas.Configuration] = None,
                    **kwargs: typing.Union[schemas.AnyTypeSchema, dict,
                                           frozendict.frozendict, str, date,
                                           datetime, uuid.UUID, int, float,
                                           decimal.Decimal, None, list, tuple,
                                           bytes],
                ) -> 'in_use':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )

            __annotations__ = {
                "id": id,
                "label": label,
                "unit": unit,
                "scope": scope,
                "max_allowed": max_allowed,
                "in_use": in_use,
            }

    unit: MetaOapg.properties.unit
    scope: 'QuotaScope'
    id: MetaOapg.properties.id
    label: MetaOapg.properties.label

    @typing.overload
    def __getitem__(
            self,
            name: typing_extensions.Literal["id"]) -> MetaOapg.properties.id:
        ...

    @typing.overload
    def __getitem__(
            self, name: typing_extensions.Literal["label"]
    ) -> MetaOapg.properties.label:
        ...

    @typing.overload
    def __getitem__(
            self, name: typing_extensions.Literal["unit"]
    ) -> MetaOapg.properties.unit:
        ...

    @typing.overload
    def __getitem__(self,
                    name: typing_extensions.Literal["scope"]) -> 'QuotaScope':
        ...

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["max_allowed"]
    ) -> MetaOapg.properties.max_allowed:
        ...

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["in_use"]
    ) -> MetaOapg.properties.in_use:
        ...

    @typing.overload
    def __getitem__(self, name: str) -> schemas.UnsetAnyTypeSchema:
        ...

    def __getitem__(self, name: typing.Union[typing_extensions.Literal[
        "id",
        "label",
        "unit",
        "scope",
        "max_allowed",
        "in_use",
    ], str]):
        # dict_instance[name] accessor
        return super().__getitem__(name)

    @typing.overload
    def get_item_oapg(
            self,
            name: typing_extensions.Literal["id"]) -> MetaOapg.properties.id:
        ...

    @typing.overload
    def get_item_oapg(
            self, name: typing_extensions.Literal["label"]
    ) -> MetaOapg.properties.label:
        ...

    @typing.overload
    def get_item_oapg(
            self, name: typing_extensions.Literal["unit"]
    ) -> MetaOapg.properties.unit:
        ...

    @typing.overload
    def get_item_oapg(self,
                      name: typing_extensions.Literal["scope"]) -> 'QuotaScope':
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["max_allowed"]
    ) -> typing.Union[MetaOapg.properties.max_allowed, schemas.Unset]:
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["in_use"]
    ) -> typing.Union[MetaOapg.properties.in_use, schemas.Unset]:
        ...

    @typing.overload
    def get_item_oapg(
            self, name: str
    ) -> typing.Union[schemas.UnsetAnyTypeSchema, schemas.Unset]:
        ...

    def get_item_oapg(self, name: typing.Union[typing_extensions.Literal[
        "id",
        "label",
        "unit",
        "scope",
        "max_allowed",
        "in_use",
    ], str]):
        return super().get_item_oapg(name)

    def __new__(
        cls,
        *_args: typing.Union[
            dict,
            frozendict.frozendict,
        ],
        unit: typing.Union[
            MetaOapg.properties.unit,
            str,
        ],
        scope: 'QuotaScope',
        id: typing.Union[
            MetaOapg.properties.id,
            str,
        ],
        label: typing.Union[
            MetaOapg.properties.label,
            str,
        ],
        max_allowed: typing.Union[MetaOapg.properties.max_allowed, dict,
                                  frozendict.frozendict, str, date, datetime,
                                  uuid.UUID, int, float, decimal.Decimal, bool,
                                  None, list, tuple, bytes, io.FileIO,
                                  io.BufferedReader,
                                  schemas.Unset] = schemas.unset,
        in_use: typing.Union[MetaOapg.properties.in_use, dict,
                             frozendict.frozendict, str, date, datetime,
                             uuid.UUID, int, float, decimal.Decimal, bool, None,
                             list, tuple, bytes, io.FileIO, io.BufferedReader,
                             schemas.Unset] = schemas.unset,
        _configuration: typing.Optional[schemas.Configuration] = None,
        **kwargs: typing.Union[schemas.AnyTypeSchema, dict,
                               frozendict.frozendict, str, date, datetime,
                               uuid.UUID, int, float, decimal.Decimal, None,
                               list, tuple, bytes],
    ) -> 'Quota':
        return super().__new__(
            cls,
            *_args,
            unit=unit,
            scope=scope,
            id=id,
            label=label,
            max_allowed=max_allowed,
            in_use=in_use,
            _configuration=_configuration,
            **kwargs,
        )


from inductiva.client.model.quota_scope import QuotaScope



# coding: utf-8

"""
    InductivaWebAPI

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""

from datetime import date, datetime  # noqa: F401
import decimal  # noqa: F401
import functools  # noqa: F401
import io  # noqa: F401
import re  # noqa: F401
import typing  # noqa: F401
import typing_extensions  # noqa: F401
import uuid  # noqa: F401

import frozendict  # noqa: F401

from inductiva.client import schemas  # noqa: F401


class TaskStatus(
    schemas.DictSchema
):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """


    class MetaOapg:
        required = {
            "id",
            "is_terminated",
            "status",
        }
        
        class properties:
            id = schemas.StrSchema
        
            @staticmethod
            def status() -> typing.Type['TaskStatusCode']:
                return TaskStatusCode
            is_terminated = schemas.BoolSchema
            
            
            class position_in_queue(
                schemas.ComposedSchema,
            ):
            
            
                class MetaOapg:
                    any_of_1 = schemas.NoneSchema
                    
                    @classmethod
                    @functools.lru_cache()
                    def any_of(cls):
                        # we need this here to make our import statements work
                        # we must store _composed_schemas in here so the code is only run
                        # when we invoke this method. If we kept this at the class
                        # level we would get an error because the class level
                        # code would be run when this module is imported, and these composed
                        # classes don't exist yet because their module has not finished
                        # loading
                        return [
                            TaskPositionInQueue,
                            cls.any_of_1,
                        ]
            
            
                def __new__(
                    cls,
                    *_args: typing.Union[dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, bool, None, list, tuple, bytes, io.FileIO, io.BufferedReader, ],
                    _configuration: typing.Optional[schemas.Configuration] = None,
                    **kwargs: typing.Union[schemas.AnyTypeSchema, dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, None, list, tuple, bytes],
                ) -> 'position_in_queue':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )
            __annotations__ = {
                "id": id,
                "status": status,
                "is_terminated": is_terminated,
                "position_in_queue": position_in_queue,
            }
    
    id: MetaOapg.properties.id
    is_terminated: MetaOapg.properties.is_terminated
    status: 'TaskStatusCode'
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["id"]) -> MetaOapg.properties.id: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["status"]) -> 'TaskStatusCode': ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["is_terminated"]) -> MetaOapg.properties.is_terminated: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["position_in_queue"]) -> MetaOapg.properties.position_in_queue: ...
    
    @typing.overload
    def __getitem__(self, name: str) -> schemas.UnsetAnyTypeSchema: ...
    
    def __getitem__(self, name: typing.Union[typing_extensions.Literal["id", "status", "is_terminated", "position_in_queue", ], str]):
        # dict_instance[name] accessor
        return super().__getitem__(name)
    
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["id"]) -> MetaOapg.properties.id: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["status"]) -> 'TaskStatusCode': ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["is_terminated"]) -> MetaOapg.properties.is_terminated: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["position_in_queue"]) -> typing.Union[MetaOapg.properties.position_in_queue, schemas.Unset]: ...
    
    @typing.overload
    def get_item_oapg(self, name: str) -> typing.Union[schemas.UnsetAnyTypeSchema, schemas.Unset]: ...
    
    def get_item_oapg(self, name: typing.Union[typing_extensions.Literal["id", "status", "is_terminated", "position_in_queue", ], str]):
        return super().get_item_oapg(name)
    

    def __new__(
        cls,
        *_args: typing.Union[dict, frozendict.frozendict, ],
        id: typing.Union[MetaOapg.properties.id, str, ],
        is_terminated: typing.Union[MetaOapg.properties.is_terminated, bool, ],
        status: 'TaskStatusCode',
        position_in_queue: typing.Union[MetaOapg.properties.position_in_queue, dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, bool, None, list, tuple, bytes, io.FileIO, io.BufferedReader, schemas.Unset] = schemas.unset,
        _configuration: typing.Optional[schemas.Configuration] = None,
        **kwargs: typing.Union[schemas.AnyTypeSchema, dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, None, list, tuple, bytes],
    ) -> 'TaskStatus':
        return super().__new__(
            cls,
            *_args,
            id=id,
            is_terminated=is_terminated,
            status=status,
            position_in_queue=position_in_queue,
            _configuration=_configuration,
            **kwargs,
        )

from inductiva.client.model.task_position_in_queue import TaskPositionInQueue
from inductiva.client.model.task_status_code import TaskStatusCode



"""Enables autocompletion for the CLI via CLI."""
import argparse

from inductiva import _cli


def enable_auto_complete(args):
    #TODO(augusto): Setup autocompletions for other shells.
    shell = args.shell
    if shell != "zsh":
        raise ValueError("At the moment inductiva supports only autocompletion "
                         "for zsh shells.")
    _cli.utils.setup_zsh_autocompletion()
    return 0


def register(parser):
    """Enable autocompletion for the cli."""

    help_message = "Enables autocomplete for shell commands."
    subparser = parser.add_parser("enable",
                                  help=help_message,
                                  formatter_class=argparse.RawTextHelpFormatter)

    subparser.description = help_message

    group = subparser.add_mutually_exclusive_group()
    group.add_argument("--shell",
                       type=str,
                       choices=["zsh"],
                       help="The shell used. Must be zsh")

    subparser.set_defaults(func=enable_auto_complete)



import typing_extensions

from inductiva.client.apis.tags import TagValues
from inductiva.client.apis.tags.compute_api import ComputeApi
from inductiva.client.apis.tags.projects_api import ProjectsApi
from inductiva.client.apis.tags.storage_api import StorageApi
from inductiva.client.apis.tags.tasks_api import TasksApi
from inductiva.client.apis.tags.users_api import UsersApi
from inductiva.client.apis.tags.version_api import VersionApi

TagToApi = typing_extensions.TypedDict(
    'TagToApi', {
        TagValues.COMPUTE: ComputeApi,
        TagValues.PROJECTS: ProjectsApi,
        TagValues.STORAGE: StorageApi,
        TagValues.TASKS: TasksApi,
        TagValues.USERS: UsersApi,
        TagValues.VERSION: VersionApi,
    })

tag_to_api = TagToApi({
    TagValues.COMPUTE: ComputeApi,
    TagValues.PROJECTS: ProjectsApi,
    TagValues.STORAGE: StorageApi,
    TagValues.TASKS: TasksApi,
    TagValues.USERS: UsersApi,
    TagValues.VERSION: VersionApi,
})



# coding: utf-8

"""
    InductivaWebAPI

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""

from datetime import date, datetime  # noqa: F401
import decimal  # noqa: F401
import functools  # noqa: F401
import io  # noqa: F401
import re  # noqa: F401
import typing  # noqa: F401
import typing_extensions  # noqa: F401
import uuid  # noqa: F401

import frozendict  # noqa: F401

from inductiva.client import schemas  # noqa: F401


class ExecuterTrackerAPIConnectionInfo(
    schemas.DictSchema
):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.

    Information sent to the executer tracker after registration.
    """


    class MetaOapg:
        required = {
            "redis_consumer_group",
            "redis_stream",
            "redis_consumer_name",
            "executer_tracker_id",
            "machine_group_id",
        }
        
        class properties:
            executer_tracker_id = schemas.UUIDSchema
            redis_stream = schemas.StrSchema
            redis_consumer_name = schemas.StrSchema
            redis_consumer_group = schemas.StrSchema
            machine_group_id = schemas.UUIDSchema
            __annotations__ = {
                "executer_tracker_id": executer_tracker_id,
                "redis_stream": redis_stream,
                "redis_consumer_name": redis_consumer_name,
                "redis_consumer_group": redis_consumer_group,
                "machine_group_id": machine_group_id,
            }
    
    redis_consumer_group: MetaOapg.properties.redis_consumer_group
    redis_stream: MetaOapg.properties.redis_stream
    redis_consumer_name: MetaOapg.properties.redis_consumer_name
    executer_tracker_id: MetaOapg.properties.executer_tracker_id
    machine_group_id: MetaOapg.properties.machine_group_id
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["executer_tracker_id"]) -> MetaOapg.properties.executer_tracker_id: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["redis_stream"]) -> MetaOapg.properties.redis_stream: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["redis_consumer_name"]) -> MetaOapg.properties.redis_consumer_name: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["redis_consumer_group"]) -> MetaOapg.properties.redis_consumer_group: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["machine_group_id"]) -> MetaOapg.properties.machine_group_id: ...
    
    @typing.overload
    def __getitem__(self, name: str) -> schemas.UnsetAnyTypeSchema: ...
    
    def __getitem__(self, name: typing.Union[typing_extensions.Literal["executer_tracker_id", "redis_stream", "redis_consumer_name", "redis_consumer_group", "machine_group_id", ], str]):
        # dict_instance[name] accessor
        return super().__getitem__(name)
    
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["executer_tracker_id"]) -> MetaOapg.properties.executer_tracker_id: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["redis_stream"]) -> MetaOapg.properties.redis_stream: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["redis_consumer_name"]) -> MetaOapg.properties.redis_consumer_name: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["redis_consumer_group"]) -> MetaOapg.properties.redis_consumer_group: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["machine_group_id"]) -> MetaOapg.properties.machine_group_id: ...
    
    @typing.overload
    def get_item_oapg(self, name: str) -> typing.Union[schemas.UnsetAnyTypeSchema, schemas.Unset]: ...
    
    def get_item_oapg(self, name: typing.Union[typing_extensions.Literal["executer_tracker_id", "redis_stream", "redis_consumer_name", "redis_consumer_group", "machine_group_id", ], str]):
        return super().get_item_oapg(name)
    

    def __new__(
        cls,
        *_args: typing.Union[dict, frozendict.frozendict, ],
        redis_consumer_group: typing.Union[MetaOapg.properties.redis_consumer_group, str, ],
        redis_stream: typing.Union[MetaOapg.properties.redis_stream, str, ],
        redis_consumer_name: typing.Union[MetaOapg.properties.redis_consumer_name, str, ],
        executer_tracker_id: typing.Union[MetaOapg.properties.executer_tracker_id, str, uuid.UUID, ],
        machine_group_id: typing.Union[MetaOapg.properties.machine_group_id, str, uuid.UUID, ],
        _configuration: typing.Optional[schemas.Configuration] = None,
        **kwargs: typing.Union[schemas.AnyTypeSchema, dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, None, list, tuple, bytes],
    ) -> 'ExecuterTrackerAPIConnectionInfo':
        return super().__new__(
            cls,
            *_args,
            redis_consumer_group=redis_consumer_group,
            redis_stream=redis_stream,
            redis_consumer_name=redis_consumer_name,
            executer_tracker_id=executer_tracker_id,
            machine_group_id=machine_group_id,
            _configuration=_configuration,
            **kwargs,
        )



"""Client for Inductiva's web API."""
import os
import sys
import logging
import contextvars
from urllib3.exceptions import MaxRetryError, NewConnectionError

from inductiva.client.apis.tags.version_api import VersionApi
from inductiva.client.configuration import Configuration
from inductiva.client.exceptions import ApiException
from inductiva._cli.cmd_user.info import get_info
from inductiva.client.api_client import ApiClient

from . import simulators
from . import resources
from . import projects
from . import commands
from . import storage
from . import utils
from . import tasks
from . import users
from . import logs
from . import api
from .templating import TemplateManager

logs.setup(getattr(logging, os.environ.get("INDUCTIVA_LOG_LEVEL", "INFO")))

api_url = os.environ.get("INDUCTIVA_API_URL", "https://api.inductiva.ai")
_output_dir = contextvars.ContextVar("INDUCTIVA_OUTPUT_DIR",
                                     default=os.environ.get(
                                         "INDUCTIVA_OUTPUT_DIR",
                                         "inductiva_output"))
_api_key = contextvars.ContextVar("INDUCTIVA_API_KEY",
                                  default=os.environ.get(
                                      "INDUCTIVA_API_KEY", None))

# Disable urllib3 warnings.
# TODO: Verify and fix the appearance of this warning.
urllib3_logger = logging.getLogger("urllib3.connectionpool")
urllib3_logger.setLevel(logging.CRITICAL)

__version__ = "0.8.6"


def set_output_dir(new_output_dir):
    """Sets the value of `inductiva._output_dir` to `new_output_dir`"""
    _output_dir.set(new_output_dir)


def get_output_dir():
    """Returns the value of inductiva._output_dir"""
    return _output_dir.get()


def _check_for_available_package_update():
    # pylint: disable=import-outside-toplevel
    from .localization import translator as __
    import urllib3
    import json

    new_version = __version__

    try:
        http = urllib3.PoolManager()
        resp = http.request("GET",
                            "https://pypi.org/pypi/inductiva/json",
                            headers={"Accept": "application/json"})
        json_response = json.loads(resp.data)
        new_version = json_response["info"]["version"]

    except Exception as ex:  # pylint: disable=broad-exception-caught
        logging.warning(__("failed-update-check", ex), exc_info=True)

    if new_version != __version__:
        msg = __("upgrade-available", new_version, __version__)
        print(msg, file=sys.stderr)


def _set_key_and_check_version():
    """Sets the api key and checks if it is valid."""
    if not utils.format_utils.getenv_bool("GITHUB_ACTIONS", False):
        set_api_key(get_api_key())

    # Perform version check only on first invocation
    if not hasattr(_set_key_and_check_version, "version_checked"):
        compare_client_and_backend_versions(__version__)
        _set_key_and_check_version.version_checked = True


_check_for_available_package_update()


def compare_client_and_backend_versions(client_version: str):
    """ Compares the provided client version 7with the backend API version.

    Sends a GET request to the backend API's version comparison endpoint
    with the client version as a parameter. Evaluates the response to
    determine if the client version is compatible with the backend version.
    Raises exceptions for communication issues or incompatibility.

    Parameters:
    - client_version (str): The version of the client to be compared with the
                            backend version.

    Raises:
    - RuntimeError: If the API cannot be reached, or if the client version is
      incompatible with the backend version, or for other general failures.
    """
    api_config = Configuration(host=api_url)

    with ApiClient(api_config) as client:
        api_instance = VersionApi(client)
        query_params = {"client_version": client_version}

        try:
            api_instance.compare_client_and_backend_versions(
                query_params=query_params)

        except (MaxRetryError, NewConnectionError) as exc:
            raise RuntimeError(
                "Failed to reach the API. "
                "Please check your connection and try again.") from exc

        except ApiException as e:
            if e.status == 406:
                raise RuntimeError(
                    f"Client version {client_version} is not compatible "
                    f"with API version {e.headers['version']}.\n"
                    "Please update the client version.") from e
            raise RuntimeError(e) from e

        except Exception as e:
            raise RuntimeError(
                f"Failed to compare client and API versions. {e}") from e


def set_api_key(api_key):
    """Sets the value of `inductiva._api_key` to `api_key"""
    if api_key is None:
        # pylint: disable=line-too-long
        raise ValueError(
            "No API Key specified. "
            "Please set the INDUCTIVA_API_KEY environment variable.\n"
            "More infomation at:"
            "https://console.inductiva.ai/")

    _api_key.set(api_key)


def get_api_key():
    """Returns the value of inductiva._api_key"""
    return _api_key.get()


def _supports_ansi():
    """Checks if we support ansi formatting for colors and bolds"""
    user_disable_ansi = utils.format_utils.getenv_bool("INDUCTIVA_DISABLE_ANSI",
                                                       False)

    if user_disable_ansi:
        return False

    if sys.platform.startswith("win"):
        return "TERM" in os.environ and os.environ["TERM"] == "xterm"

    if is_notebook():
        return True

    return hasattr(sys.stdout, "isatty") and sys.stdout.isatty()


def is_notebook():
    try:
        ipython = __import__("IPython")
        if ipython.get_ipython() is None:
            return False
    except ImportError:
        return False
    return True


def _check_user_info():

    if utils.format_utils.getenv_bool("GITHUB_ACTIONS",False) or \
       utils.format_utils.getenv_bool("INDUCTIVA_DISABLE_IMPORT_INFO",False):
        return

    if not logs.is_cli():
        # Determine if we are importing from cli or script file
        # and only print info if called from a script file
        get_info(None, sys.stdout)


ansi_enabled = _supports_ansi()

_set_key_and_check_version()

_check_user_info()



"""CaNS module of the API for numerical simulations of fluid flows."""
from typing import Optional

from inductiva import types, tasks, simulators


@simulators.simulator.mpi_enabled
class CaNS(simulators.Simulator):
    """Class to invoke a generic CaNS simulation on the API.

    """

    def __init__(self, /, version: Optional[str] = None, use_dev: bool = False):
        """Initialize the CaNS simulator.
        
        Args:
            version (str): The version of the simulator to use. If None, the
                latest available version in the platform is used.
            use_dev (bool): Request use of the development version of
                the simulator. By default (False), the production version
                is used.
        """
        super().__init__(version=version, use_dev=use_dev)
        self.api_method_name = "cans.cans.run_simulation"

    def run(self,
            input_dir: str,
            sim_config_filename: str,
            use_hwthread: bool = True,
            n_vcpus: Optional[int] = None,
            extra_metadata: Optional[dict] = None,
            storage_dir: Optional[str] = "",
            resubmit_on_preemption: bool = False,
            on: Optional[types.ComputationalResources] = None,
            **kwargs) -> tasks.Task:
        """Run the simulation.

        Args:
            input_dir: Path to the directory of the simulation input files.
            n_vcpus: Number of vCPUs to use in the simulation. If not provided
            (default), all vCPUs will be used.
            use_hwthread: If specified Open MPI will attempt to discover the
            number of hardware threads on the node, and use that as the
            number of slots available.
            on: The computational resource to launch the simulation on. If None
                the simulation is submitted to a machine in the default pool.
            resubmit_on_preemption (bool): Resubmit task for execution when
                previous execution attempts were preempted. Only applicable when
                using a preemptible resource, i.e., resource instantiates with
                `spot=True`.
            other arguments: See the documentation of the base class.
        """
        return super().run(input_dir,
                           on=on,
                           n_vcpus=n_vcpus,
                           storage_dir=storage_dir,
                           use_hwthread=use_hwthread,
                           extra_metadata=extra_metadata,
                           input_filename=sim_config_filename,
                           resubmit_on_preemption=resubmit_on_preemption,
                           **kwargs)



# coding: utf-8
"""
    InductivaWebAPI

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""

from datetime import date, datetime  # noqa: F401
import decimal  # noqa: F401
import functools  # noqa: F401
import io  # noqa: F401
import re  # noqa: F401
import typing  # noqa: F401
import typing_extensions  # noqa: F401
import uuid  # noqa: F401

import frozendict  # noqa: F401

from inductiva.client import schemas  # noqa: F401


class User(schemas.DictSchema):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    class MetaOapg:
        required = {
            "terms_and_conditions_decision",
            "campaigns",
            "tier",
            "total_available_credits",
            "email",
            "username",
        }

        class properties:
            username = schemas.StrSchema
            email = schemas.StrSchema

            @staticmethod
            def terms_and_conditions_decision(
            ) -> typing.Type['TermsAndConditions']:
                return TermsAndConditions

            total_available_credits = schemas.NumberSchema

            @staticmethod
            def tier() -> typing.Type['Tier']:
                return Tier

            class campaigns(schemas.ListSchema):

                class MetaOapg:

                    @staticmethod
                    def items() -> typing.Type['UserCampaign']:
                        return UserCampaign

                def __new__(
                    cls,
                    _arg: typing.Union[typing.Tuple['UserCampaign'],
                                       typing.List['UserCampaign']],
                    _configuration: typing.Optional[
                        schemas.Configuration] = None,
                ) -> 'campaigns':
                    return super().__new__(
                        cls,
                        _arg,
                        _configuration=_configuration,
                    )

                def __getitem__(self, i: int) -> 'UserCampaign':
                    return super().__getitem__(i)

            class name(
                    schemas.ComposedSchema,):

                class MetaOapg:
                    any_of_0 = schemas.StrSchema
                    any_of_1 = schemas.NoneSchema

                    @classmethod
                    @functools.lru_cache()
                    def any_of(cls):
                        # we need this here to make our import statements work
                        # we must store _composed_schemas in here so the code is only run
                        # when we invoke this method. If we kept this at the class
                        # level we would get an error because the class level
                        # code would be run when this module is imported, and these composed
                        # classes don't exist yet because their module has not finished
                        # loading
                        return [
                            cls.any_of_0,
                            cls.any_of_1,
                        ]

                def __new__(
                    cls,
                    *_args: typing.Union[
                        dict,
                        frozendict.frozendict,
                        str,
                        date,
                        datetime,
                        uuid.UUID,
                        int,
                        float,
                        decimal.Decimal,
                        bool,
                        None,
                        list,
                        tuple,
                        bytes,
                        io.FileIO,
                        io.BufferedReader,
                    ],
                    _configuration: typing.Optional[
                        schemas.Configuration] = None,
                    **kwargs: typing.Union[schemas.AnyTypeSchema, dict,
                                           frozendict.frozendict, str, date,
                                           datetime, uuid.UUID, int, float,
                                           decimal.Decimal, None, list, tuple,
                                           bytes],
                ) -> 'name':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )

            class terms_and_conditions_decision_ts(
                    schemas.DateTimeBase,
                    schemas.ComposedSchema,
            ):

                class MetaOapg:
                    format = 'date-time'
                    any_of_0 = schemas.StrSchema
                    any_of_1 = schemas.NoneSchema

                    @classmethod
                    @functools.lru_cache()
                    def any_of(cls):
                        # we need this here to make our import statements work
                        # we must store _composed_schemas in here so the code is only run
                        # when we invoke this method. If we kept this at the class
                        # level we would get an error because the class level
                        # code would be run when this module is imported, and these composed
                        # classes don't exist yet because their module has not finished
                        # loading
                        return [
                            cls.any_of_0,
                            cls.any_of_1,
                        ]

                def __new__(
                    cls,
                    *_args: typing.Union[
                        dict,
                        frozendict.frozendict,
                        str,
                        date,
                        datetime,
                        uuid.UUID,
                        int,
                        float,
                        decimal.Decimal,
                        bool,
                        None,
                        list,
                        tuple,
                        bytes,
                        io.FileIO,
                        io.BufferedReader,
                    ],
                    _configuration: typing.Optional[
                        schemas.Configuration] = None,
                    **kwargs: typing.Union[schemas.AnyTypeSchema, dict,
                                           frozendict.frozendict, str, date,
                                           datetime, uuid.UUID, int, float,
                                           decimal.Decimal, None, list, tuple,
                                           bytes],
                ) -> 'terms_and_conditions_decision_ts':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )

            credits_currency = schemas.StrSchema
            __annotations__ = {
                "username":
                    username,
                "email":
                    email,
                "terms_and_conditions_decision":
                    terms_and_conditions_decision,
                "total_available_credits":
                    total_available_credits,
                "tier":
                    tier,
                "campaigns":
                    campaigns,
                "name":
                    name,
                "terms_and_conditions_decision_ts":
                    terms_and_conditions_decision_ts,
                "credits_currency":
                    credits_currency,
            }

    terms_and_conditions_decision: 'TermsAndConditions'
    campaigns: MetaOapg.properties.campaigns
    tier: 'Tier'
    total_available_credits: MetaOapg.properties.total_available_credits
    email: MetaOapg.properties.email
    username: MetaOapg.properties.username

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["username"]
    ) -> MetaOapg.properties.username:
        ...

    @typing.overload
    def __getitem__(
            self, name: typing_extensions.Literal["email"]
    ) -> MetaOapg.properties.email:
        ...

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["terms_and_conditions_decision"]
    ) -> 'TermsAndConditions':
        ...

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["total_available_credits"]
    ) -> MetaOapg.properties.total_available_credits:
        ...

    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["tier"]) -> 'Tier':
        ...

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["campaigns"]
    ) -> MetaOapg.properties.campaigns:
        ...

    @typing.overload
    def __getitem__(
            self, name: typing_extensions.Literal["name"]
    ) -> MetaOapg.properties.name:
        ...

    @typing.overload
    def __getitem__(
        self,
        name: typing_extensions.Literal["terms_and_conditions_decision_ts"]
    ) -> MetaOapg.properties.terms_and_conditions_decision_ts:
        ...

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["credits_currency"]
    ) -> MetaOapg.properties.credits_currency:
        ...

    @typing.overload
    def __getitem__(self, name: str) -> schemas.UnsetAnyTypeSchema:
        ...

    def __getitem__(self, name: typing.Union[typing_extensions.Literal[
        "username",
        "email",
        "terms_and_conditions_decision",
        "total_available_credits",
        "tier",
        "campaigns",
        "name",
        "terms_and_conditions_decision_ts",
        "credits_currency",
    ], str]):
        # dict_instance[name] accessor
        return super().__getitem__(name)

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["username"]
    ) -> MetaOapg.properties.username:
        ...

    @typing.overload
    def get_item_oapg(
            self, name: typing_extensions.Literal["email"]
    ) -> MetaOapg.properties.email:
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["terms_and_conditions_decision"]
    ) -> 'TermsAndConditions':
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["total_available_credits"]
    ) -> MetaOapg.properties.total_available_credits:
        ...

    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["tier"]) -> 'Tier':
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["campaigns"]
    ) -> MetaOapg.properties.campaigns:
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["name"]
    ) -> typing.Union[MetaOapg.properties.name, schemas.Unset]:
        ...

    @typing.overload
    def get_item_oapg(
        self,
        name: typing_extensions.Literal["terms_and_conditions_decision_ts"]
    ) -> typing.Union[MetaOapg.properties.terms_and_conditions_decision_ts,
                      schemas.Unset]:
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["credits_currency"]
    ) -> typing.Union[MetaOapg.properties.credits_currency, schemas.Unset]:
        ...

    @typing.overload
    def get_item_oapg(
            self, name: str
    ) -> typing.Union[schemas.UnsetAnyTypeSchema, schemas.Unset]:
        ...

    def get_item_oapg(self, name: typing.Union[typing_extensions.Literal[
        "username",
        "email",
        "terms_and_conditions_decision",
        "total_available_credits",
        "tier",
        "campaigns",
        "name",
        "terms_and_conditions_decision_ts",
        "credits_currency",
    ], str]):
        return super().get_item_oapg(name)

    def __new__(
        cls,
        *_args: typing.Union[
            dict,
            frozendict.frozendict,
        ],
        terms_and_conditions_decision: 'TermsAndConditions',
        campaigns: typing.Union[
            MetaOapg.properties.campaigns,
            list,
            tuple,
        ],
        tier: 'Tier',
        total_available_credits: typing.Union[
            MetaOapg.properties.total_available_credits,
            decimal.Decimal,
            int,
            float,
        ],
        email: typing.Union[
            MetaOapg.properties.email,
            str,
        ],
        username: typing.Union[
            MetaOapg.properties.username,
            str,
        ],
        name: typing.Union[MetaOapg.properties.name, dict,
                           frozendict.frozendict, str, date, datetime,
                           uuid.UUID, int, float, decimal.Decimal, bool, None,
                           list, tuple, bytes, io.FileIO, io.BufferedReader,
                           schemas.Unset] = schemas.unset,
        terms_and_conditions_decision_ts: typing.Union[
            MetaOapg.properties.terms_and_conditions_decision_ts, dict,
            frozendict.frozendict, str, date, datetime, uuid.UUID, int, float,
            decimal.Decimal, bool, None, list, tuple, bytes, io.FileIO,
            io.BufferedReader, schemas.Unset] = schemas.unset,
        credits_currency: typing.Union[MetaOapg.properties.credits_currency,
                                       str, schemas.Unset] = schemas.unset,
        _configuration: typing.Optional[schemas.Configuration] = None,
        **kwargs: typing.Union[schemas.AnyTypeSchema, dict,
                               frozendict.frozendict, str, date, datetime,
                               uuid.UUID, int, float, decimal.Decimal, None,
                               list, tuple, bytes],
    ) -> 'User':
        return super().__new__(
            cls,
            *_args,
            terms_and_conditions_decision=terms_and_conditions_decision,
            campaigns=campaigns,
            tier=tier,
            total_available_credits=total_available_credits,
            email=email,
            username=username,
            name=name,
            terms_and_conditions_decision_ts=terms_and_conditions_decision_ts,
            credits_currency=credits_currency,
            _configuration=_configuration,
            **kwargs,
        )


from inductiva.client.model.terms_and_conditions import TermsAndConditions
from inductiva.client.model.tier import Tier
from inductiva.client.model.user_campaign import UserCampaign



"""Launch simulation resources via CLI."""

import argparse
from inductiva import resources


def start_machine_group(args):
    """Start a resource."""
    machine_type = args.machine_type
    num_machines = args.num_machines
    data_disk_gb = args.disk_size
    spot = args.spot

    machine = resources.MachineGroup(machine_type=machine_type,
                                     num_machines=num_machines,
                                     data_disk_gb=data_disk_gb,
                                     spot=spot)

    machine.start()
    print(f"{repr(machine)} started.")


def register(parser):
    """Register the launch of resources commands."""

    subparser = parser.add_parser("start",
                                  help="Start computational resources.",
                                  formatter_class=argparse.RawTextHelpFormatter)

    subparser.description = (
        "The `inductiva resources start` command initiates "
        "computational resources.\n"
        "It provides options to specify the machine type, "
        "number of machines,"
        "disk size in GB, and the use of spot instances.\n"
        "This command is suitable"
        "for initiating both individual machines and clusters.\n\n"
        "To terminate resources, use the `inductiva resources"
        " terminate` command.\n")
    subparser.add_argument("machine_type",
                           type=str,
                           help="Machine type to start.")
    subparser.add_argument("-n",
                           "--num_machines",
                           default=1,
                           type=int,
                           help="Number of machines to start.")
    subparser.add_argument("-d",
                           "--data_disk_gb",
                           default=70,
                           type=int,
                           help="Disk size in GB.")
    subparser.add_argument("-s",
                           "--spot",
                           default=False,
                           action="store_true",
                           help="Whether to use spot instances.")

    subparser.set_defaults(func=start_machine_group)



"""Dummy simulator for demonstrating and testing purposes.

This simulator acts as an echo of the arguments passed and can be used to
test the commands passed to the simulator.
"""

from typing import Optional

from inductiva import types, tasks, simulators


class DummySimulator(simulators.Simulator):
    """Dummy simulator for demo and testing purposes."""

    def __init__(self, /, version: Optional[str] = None, use_dev: bool = False):
        """Initialize the simulator.

        Args:
            version (str): The version of the simulator to use. If None, the
                latest available version in the platform is used.
            use_dev (bool): Request use of the development version of
                the simulator. By default (False), the production version
                is used.
        """
        super().__init__(version=version, use_dev=use_dev)
        self.api_method_name = "tester.echo.run_simulation"

    def _get_image_uri(self):
        return None

    def run(self,
            input_dir: str,
            input_filename: str,
            sleep_time: Optional[float] = 1,
            on: Optional[types.ComputationalResources] = None,
            storage_dir: Optional[str] = "",
            resubmit_on_preemption: bool = False,
            extra_metadata: Optional[dict] = None,
            **kwargs) -> tasks.Task:
        """Run a dummy simulation that echo's to a file.

        Args:
            input_dir: Path to directory with simulation input files.
            input_filename: Name of the test input file.
            sleep_time: Time to sleep before running the commands.
            extra_metadata: Extra metadata to be sent to the backend.
            resubmit_on_preemption (bool): Resubmit task for execution when
                previous execution attempts were preempted. Only applicable when
                using a preemptible resource, i.e., resource instantiates with
                `spot=True`.
        """

        return super().run(input_dir,
                           on=on,
                           input_filename=input_filename,
                           sleep_time=sleep_time,
                           storage_dir=storage_dir,
                           extra_metadata=extra_metadata,
                           resubmit_on_preemption=resubmit_on_preemption,
                           **kwargs)



# coding: utf-8
"""
    InductivaWebAPI

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""

from datetime import date, datetime  # noqa: F401
import decimal  # noqa: F401
import functools  # noqa: F401
import io  # noqa: F401
import re  # noqa: F401
import typing  # noqa: F401
import typing_extensions  # noqa: F401
import uuid  # noqa: F401

import frozendict  # noqa: F401

from inductiva.client import schemas  # noqa: F401


class UserTaskStatusOverview(schemas.DictSchema):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    class MetaOapg:
        required = {
            "status_counts",
        }

        class properties:

            class status_counts(schemas.DictSchema):

                class MetaOapg:
                    additional_properties = schemas.IntSchema

                def __getitem__(
                        self, name: typing.Union[
                            str,
                        ]) -> MetaOapg.additional_properties:
                    # dict_instance[name] accessor
                    return super().__getitem__(name)

                def get_item_oapg(
                        self, name: typing.Union[
                            str,
                        ]) -> MetaOapg.additional_properties:
                    return super().get_item_oapg(name)

                def __new__(
                    cls,
                    *_args: typing.Union[
                        dict,
                        frozendict.frozendict,
                    ],
                    _configuration: typing.Optional[
                        schemas.Configuration] = None,
                    **kwargs: typing.Union[
                        MetaOapg.additional_properties,
                        decimal.Decimal,
                        int,
                    ],
                ) -> 'status_counts':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )

            __annotations__ = {
                "status_counts": status_counts,
            }

    status_counts: MetaOapg.properties.status_counts

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["status_counts"]
    ) -> MetaOapg.properties.status_counts:
        ...

    @typing.overload
    def __getitem__(self, name: str) -> schemas.UnsetAnyTypeSchema:
        ...

    def __getitem__(self,
                    name: typing.Union[typing_extensions.Literal[
                        "status_counts",
                    ], str]):
        # dict_instance[name] accessor
        return super().__getitem__(name)

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["status_counts"]
    ) -> MetaOapg.properties.status_counts:
        ...

    @typing.overload
    def get_item_oapg(
            self, name: str
    ) -> typing.Union[schemas.UnsetAnyTypeSchema, schemas.Unset]:
        ...

    def get_item_oapg(self,
                      name: typing.Union[typing_extensions.Literal[
                          "status_counts",
                      ], str]):
        return super().get_item_oapg(name)

    def __new__(
        cls,
        *_args: typing.Union[
            dict,
            frozendict.frozendict,
        ],
        status_counts: typing.Union[
            MetaOapg.properties.status_counts,
            dict,
            frozendict.frozendict,
        ],
        _configuration: typing.Optional[schemas.Configuration] = None,
        **kwargs: typing.Union[schemas.AnyTypeSchema, dict,
                               frozendict.frozendict, str, date, datetime,
                               uuid.UUID, int, float, decimal.Decimal, None,
                               list, tuple, bytes],
    ) -> 'UserTaskStatusOverview':
        return super().__new__(
            cls,
            *_args,
            status_counts=status_counts,
            _configuration=_configuration,
            **kwargs,
        )



# coding: utf-8

"""
    InductivaWebAPI

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""

from datetime import date, datetime  # noqa: F401
import decimal  # noqa: F401
import functools  # noqa: F401
import io  # noqa: F401
import re  # noqa: F401
import typing  # noqa: F401
import typing_extensions  # noqa: F401
import uuid  # noqa: F401

import frozendict  # noqa: F401

from inductiva.client import schemas  # noqa: F401


class MachineGroupType(
    schemas.EnumBase,
    schemas.StrSchema
):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.

    Possible Machine Group types.

This enum is also created in Postgres.
    """
    
    @schemas.classproperty
    def STANDARD(cls):
        return cls("standard")
    
    @schemas.classproperty
    def MPI(cls):
        return cls("mpi")
    
    @schemas.classproperty
    def ELASTIC(cls):
        return cls("elastic")



# do not import all endpoints into this module because that uses a lot of memory and stack frames
# if you need the ability to import all endpoints from this module, import them with
# from inductiva.client.apis.path_to_api import path_to_api

import enum


class PathValues(str, enum.Enum):
    TASKS_AUTH = "/tasks/auth"
    TASKS_SUBMIT = "/tasks/submit"
    TASKS_TASK_ID_INPUT_UPLOAD_URL = "/tasks/{task_id}/input_upload_url"
    TASKS_TASK_ID_INPUT_UPLOADED = "/tasks/{task_id}/input_uploaded"
    TASKS_TASK_ID_INPUT = "/tasks/{task_id}/input"
    TASKS_TASK_ID = "/tasks/{task_id}"
    TASKS = "/tasks"
    TASKS_TASK_ID_STATUS = "/tasks/{task_id}/status"
    TASKS_TASK_ID_POSITION_IN_QUEUE = "/tasks/{task_id}/position_in_queue"
    TASKS_TASK_ID_OUTPUT_LIST = "/tasks/{task_id}/output/list"
    TASKS_TASK_ID_DOWNLOAD_OUTPUT_URL = "/tasks/{task_id}/download_output_url"
    TASKS_TASK_ID_OUTPUT = "/tasks/{task_id}/output"
    TASKS_TASK_ID_RESUBMIT = "/tasks/{task_id}/resubmit"
    TASKS_TASK_ID_KILL = "/tasks/{task_id}/kill"
    TASKS_TASK_ID_DISABLE_LOGS = "/tasks/{task_id}/disable_logs"
    TASKS_TASK_ID_FILES = "/tasks/{task_id}/files"
    ADMIN_USERS = "/admin/users"
    ADMIN_USERS_EMAIL_TERMS_AND_CONDITIONS = "/admin/users/{email}/terms_and_conditions"
    ADMIN_USERS_EMAIL_API_KEY = "/admin/users/{email}/api_key"
    ADMIN_USERS_EMAIL = "/admin/users/{email}"
    ADMIN_USERS_EMAIL_CAMPAIGN_CAMPAIGN_ID = "/admin/users/{email}/campaign/{campaign_id}"
    ADMIN_USERS_USERNAME_STORAGE_SIZE_FS = "/admin/users/{username}/storage_size_fs"
    ADMIN_USERS_USERNAME_STORAGE_SIZE = "/admin/users/{username}/storage_size"
    ADMIN_USERS_USERNAME_TASKS = "/admin/users/{username}/tasks"
    ADMIN_GROUPS = "/admin/groups"
    ADMIN_GROUPS_ACTIVE = "/admin/groups/active"
    ADMIN_GROUPS_DEFAULT = "/admin/groups/default"
    ADMIN_GROUPS_DEFAULT_MACHINE_GROUP_ID = "/admin/groups/default/{machine_group_id}"
    ADMIN_PROVIDERS = "/admin/providers"
    ADMIN_PROVIDERS_PROVIDER_ID = "/admin/providers/{provider_id}"
    ADMIN_ACTIVE_TASKS = "/admin/active_tasks"
    ADMIN_EXECUTERTRACKER_TOKEN = "/admin/executer-tracker/token"
    ADMIN_GROUPS_MACHINE_GROUP_ID_TERMINATE = "/admin/groups/{machine_group_id}/terminate"
    ADMIN_CAMPAIGNS = "/admin/campaigns"
    ADMIN_CAMPAIGNS_CAMPAIGN_ID = "/admin/campaigns/{campaign_id}"
    ADMIN_CAMPAIGNS_CAMPAIGN_ID_DEACTIVATE = "/admin/campaigns/{campaign_id}/deactivate"
    ADMIN_CAMPAIGNS_CAMPAIGN_ID_USERS = "/admin/campaigns/{campaign_id}/users"
    ADMIN_CAMPAIGNS_CAMPAIGN_ID_CAPABILITIES = "/admin/campaigns/{campaign_id}/capabilities"
    ADMIN_CAMPAIGNS_CAMPAIGN_ID_CAPABILITIES_CAPABILITY_ID = "/admin/campaigns/{campaign_id}/capabilities/{capability_id}"
    ADMIN_CAMPAIGNS_CAMPAIGN_ID_QUOTAS = "/admin/campaigns/{campaign_id}/quotas"
    ADMIN_CAMPAIGNS_CAMPAIGN_ID_QUOTAS_QUOTA_ID = "/admin/campaigns/{campaign_id}/quotas/{quota_id}"
    EXECUTERTRACKER_REGISTER = "/executer-tracker/register"
    EXECUTERTRACKER_MACHINE_ID = "/executer-tracker/{machine_id}"
    EXECUTERTRACKER_MACHINE_ID_TASK = "/executer-tracker/{machine_id}/task"
    EXECUTERTRACKER_MACHINE_ID_TASK_TASK_ID_MESSAGE = "/executer-tracker/{machine_id}/task/{task_id}/message"
    EXECUTERTRACKER_MACHINE_ID_TASK_TASK_ID_MESSAGE_UNBLOCK = "/executer-tracker/{machine_id}/task/{task_id}/message/unblock"
    EXECUTERTRACKER_MACHINE_ID_EVENT = "/executer-tracker/{machine_id}/event"
    EXECUTERTRACKER_MACHINE_ID_TASK_TASK_ID_DOWNLOAD_INPUT_URL = "/executer-tracker/{machine_id}/task/{task_id}/download_input_url"
    EXECUTERTRACKER_MACHINE_ID_TASK_TASK_ID_UPLOAD_OUTPUT_URL = "/executer-tracker/{machine_id}/task/{task_id}/upload_output_url"
    EXECUTERTRACKER_MACHINE_ID_TASK_TASK_ID_METRIC = "/executer-tracker/{machine_id}/task/{task_id}/metric"
    EXECUTERTRACKER_MACHINE_ID_RESIZE_DISK = "/executer-tracker/{machine_id}/resize_disk"
    COMPUTE_GROUP = "/compute/group"
    COMPUTE_TYPE = "/compute/type"
    COMPUTE_GROUP_START = "/compute/group/start"
    COMPUTE_PRICE = "/compute/price"
    COMPUTE_GROUPS = "/compute/groups"
    COMPUTE_GROUP_STATUS = "/compute/group_status"
    COMPUTE_MACHINE_TYPES = "/compute/machine_types"
    COMPUTE_GROUP_NAME = "/compute/group/{name}"
    STORAGE_SIZE = "/storage/size"
    STORAGE_CONTENTS = "/storage/contents"
    VERSION = "/version"
    VERSIONCHECK = "/version-check"
    USERS_QUOTAS = "/users/quotas"
    USERS_INFO = "/users/info"
    PROJECTS = "/projects"
    PROJECTS_NAME = "/projects/{name}"
    METRICS_USERS_USERNAME_ACTIVITY = "/metrics/users/{username}/activity"
    METRICS_USERS_USERNAME_COST_OVER_TIME = "/metrics/users/{username}/cost_over_time"
    METRICS_USERS_USERNAME_TASK_STATUS_OVERVIEW = "/metrics/users/{username}/task_status_overview"
    METRICS_USERS_USERNAME_COMPUTATION_TIME_TREND = "/metrics/users/{username}/computation_time_trend"
    METRICS_USERS_USERNAME_TASKS_OVERVIEW = "/metrics/users/{username}/tasks_overview"
    METRICS_USERS_USERNAME_MOST_USED_MACHINE_TYPES = "/metrics/users/{username}/most_used_machine_types"
    METRICS_USERS_USERNAME_MOST_USED_SIMULATORS_OVERVIEW = "/metrics/users/{username}/most_used_simulators_overview"



# coding: utf-8
"""
    InductivaWebAPI

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""

from datetime import date, datetime  # noqa: F401
import decimal  # noqa: F401
import functools  # noqa: F401
import io  # noqa: F401
import re  # noqa: F401
import typing  # noqa: F401
import typing_extensions  # noqa: F401
import uuid  # noqa: F401

import frozendict  # noqa: F401

from inductiva.client import schemas  # noqa: F401


class Campaign(schemas.DictSchema):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    class MetaOapg:
        required = {
            "end_date",
            "credits",
            "begin_date",
            "id",
            "label",
        }

        class properties:
            id = schemas.StrSchema
            label = schemas.StrSchema
            credits = schemas.NumberSchema
            begin_date = schemas.DateTimeSchema
            end_date = schemas.DateTimeSchema
            __annotations__ = {
                "id": id,
                "label": label,
                "credits": credits,
                "begin_date": begin_date,
                "end_date": end_date,
            }

    end_date: MetaOapg.properties.end_date
    credits: MetaOapg.properties.credits
    begin_date: MetaOapg.properties.begin_date
    id: MetaOapg.properties.id
    label: MetaOapg.properties.label

    @typing.overload
    def __getitem__(
            self,
            name: typing_extensions.Literal["id"]) -> MetaOapg.properties.id:
        ...

    @typing.overload
    def __getitem__(
            self, name: typing_extensions.Literal["label"]
    ) -> MetaOapg.properties.label:
        ...

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["credits"]
    ) -> MetaOapg.properties.credits:
        ...

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["begin_date"]
    ) -> MetaOapg.properties.begin_date:
        ...

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["end_date"]
    ) -> MetaOapg.properties.end_date:
        ...

    @typing.overload
    def __getitem__(self, name: str) -> schemas.UnsetAnyTypeSchema:
        ...

    def __getitem__(self, name: typing.Union[typing_extensions.Literal[
        "id",
        "label",
        "credits",
        "begin_date",
        "end_date",
    ], str]):
        # dict_instance[name] accessor
        return super().__getitem__(name)

    @typing.overload
    def get_item_oapg(
            self,
            name: typing_extensions.Literal["id"]) -> MetaOapg.properties.id:
        ...

    @typing.overload
    def get_item_oapg(
            self, name: typing_extensions.Literal["label"]
    ) -> MetaOapg.properties.label:
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["credits"]
    ) -> MetaOapg.properties.credits:
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["begin_date"]
    ) -> MetaOapg.properties.begin_date:
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["end_date"]
    ) -> MetaOapg.properties.end_date:
        ...

    @typing.overload
    def get_item_oapg(
            self, name: str
    ) -> typing.Union[schemas.UnsetAnyTypeSchema, schemas.Unset]:
        ...

    def get_item_oapg(self, name: typing.Union[typing_extensions.Literal[
        "id",
        "label",
        "credits",
        "begin_date",
        "end_date",
    ], str]):
        return super().get_item_oapg(name)

    def __new__(
        cls,
        *_args: typing.Union[
            dict,
            frozendict.frozendict,
        ],
        end_date: typing.Union[
            MetaOapg.properties.end_date,
            str,
            datetime,
        ],
        credits: typing.Union[
            MetaOapg.properties.credits,
            decimal.Decimal,
            int,
            float,
        ],
        begin_date: typing.Union[
            MetaOapg.properties.begin_date,
            str,
            datetime,
        ],
        id: typing.Union[
            MetaOapg.properties.id,
            str,
        ],
        label: typing.Union[
            MetaOapg.properties.label,
            str,
        ],
        _configuration: typing.Optional[schemas.Configuration] = None,
        **kwargs: typing.Union[schemas.AnyTypeSchema, dict,
                               frozendict.frozendict, str, date, datetime,
                               uuid.UUID, int, float, decimal.Decimal, None,
                               list, tuple, bytes],
    ) -> 'Campaign':
        return super().__new__(
            cls,
            *_args,
            end_date=end_date,
            credits=credits,
            begin_date=begin_date,
            id=id,
            label=label,
            _configuration=_configuration,
            **kwargs,
        )
